/**
 * * Copyright (c) 2018. Honeywell International, Inc. All Rights Reserved.
 */

/**
 * A module defining `SbcOfflineEngineeringWizard`.
 *
 * @module nmodule/sbcOfflineEngineering/rc/SbcOfflineEngineeringWizard
 */
define(['bajaux/Widget',
    'bajaux/mixin/subscriberMixIn',
    'jquery',
    'Promise',
    'underscore',
    'nmodule/sbcOfflineEngineering/rc/js/util/utils',
    'nmodule/sbcOfflineEngineering/rc/js/offlineEngineeringSelectView',
    'nmodule/sbcOfflineEngineering/rc/js/offlineEngineeringScanView',
    'nmodule/sbcOfflineEngineering/rc/js/offlineEngineeringDatapointsView',
    'nmodule/sbcOfflineEngineering/rc/js/offlineEngineeringAlarmlistView',
    'nmodule/sbcOfflineEngineering/rc/js/offlineEngineeringHdlogView',
    'nmodule/sbcOfflineEngineering/rc/js/offlineEngineeringImportView',
    'nmodule/sbcOfflineEngineering/rc/js/offlineEngineeringFinishView',
    'hbs!nmodule/sbcOfflineEngineering/rc/hbs/offlineEngineeringTemplate',
    'lex!sbcOfflineEngineering',
    'css!nmodule/sbcOfflineEngineering/rc/css/sbcOfflineEngineering'], function (
    Widget,
    subscriberMixin,
    $,
    Promise,
    _,
    utils,
    selectView,
    scanView,
    datapointsView,
    alarmlistView,
    hdlogView,
    importView,
    finishView,
    template,
    lexicons) {

    'use strict';

    /**
     * A demonstration Widget. This builds a list of buttons from the slots of a
     * Complex value, allowing the user to select a slot.
     *
     * @class
     * @extends module:bajaux/Widget
     * @alias module:nmodule/sbcOfflineEngineering/rc/SbcOfflineEngineeringWizard
     */
    var SbcOfflineEngineeringWizard = function SbcOfflineEngineeringWizard() {
        /** remember to call super constructor. Javascript won't do this for you */
        var _self = this;
        Widget.apply(_self, arguments);
        subscriberMixin(_self);

        _self.currentView = null;

        /**
         * perform cleanup for current view and load next view into the wizard
         */

        _self.nextView = function() {
            return new Promise(function (resolve) {

                var currentView = _self.currentView;
                //TODO: update params with required parameters between screens
                var params = {subscriber:_self.getSubscriber(), component : _self.component};
                var validInputs = true;

                //Note: Not all screens need input validation
                if(currentView) {
                    if(currentView.validate) {
                        currentView.validate()
                            .then(function(validInputs){
                                if(validInputs) {
                                    currentView.next(event, params)
                                        .then(function() {
                                            $.extend(true, _self.soeData, currentView.getData());
                                            _self.loadNextView(_self.getUIView(currentView, "next"));
                                            resolve();
                                        });
                                }
                            });
                    } else {
                        currentView.next(event, params)
                            .then(function() {
                                $.extend(true, _self.soeData, currentView.getData());
                                _self.loadNextView(_self.getUIView(currentView, "next"));
                                resolve();
                            });
                    }
                }
            });

        };

        /**
         * load next view into the wizard
         * @param  nextView: the next view to be loaded into the widget
         *
         */

        _self.updateSelections = function(view){
                if((view == datapointsView)
                || (view == alarmlistView)
                || (view == hdlogView)
                || (view == importView)
                || (view == finishView)) {
                    if(_self.soeData.scannedModules.datapoints == 0){
                        _self.soeData.selectedModules.datapoints = false;
                    }

                    if(_self.soeData.scannedModules.alarms == 0){
                        _self.soeData.selectedModules.alarms = false;
                    }

                    if(_self.soeData.scannedModules.hdlogs == 0){
                        _self.soeData.selectedModules.hdlogs = false;
                    }

                }
        }
        _self.loadNextView = function(nextView) {
            var dom = _self.jq();
            var params, contentDom, viewID, existView
            _self.updateSelections(nextView);
            $(dom).find(".sbcOfflineEngineering-steps .active-step").addClass("done");

            _self.doUpdateView(dom, nextView.params, null);

            _self.currentView = nextView;
            params = {subscriber:_self.getSubscriber(), component : _self.component, widgetContainer:dom};
            $.extend(true, params, _self.soeData);

            contentDom = (dom).find("#sbcOfflineEngineering-contentWrapper");
            viewID = nextView.params.id;
            if (viewID !== undefined) {
                existView = $(contentDom).children("#" + viewID);
                if(existView.length > 0) {
                    //remove all the next siblings if any... doesn't make sense to have them if are going to replace them anyway
                    $(existView).nextAll().remove();
                    $(contentDom).children("#" + viewID).remove();
                }
            }
            _self.currentView.loadView(contentDom, params);
        };

        /**
         * perform cleanup for current view and load previous view into the wizard
         *
         */

        _self.prevView = function() {
            var currentView = _self.currentView;
            var params = {subscriber:_self.getSubscriber(), component: _self.component};

            if(currentView) {
                currentView.back(event, params)
                    .then(function() {
                        _self.loadPrevView(_self.getUIView(currentView, "prev"));
                    });
            }
        };

        /**
         * load previous view into the wizard
         * @param  prevView: the previous view to be loaded into the widget
         *
         */

        _self.loadPrevView = function(prevView) {
            var actionBttnsStatus = null;
            var dom = _self.jq();
            _self.currentView = prevView;
            if(prevView.enableDisableActionBttns) {
                actionBttnsStatus = prevView.enableDisableActionBttns();
            }
//            $(dom).find(".sbcOfflineEngineering-steps  .active-step").addClass("default");
            _self.doUpdateView(dom, prevView.params, actionBttnsStatus);
            $(dom).find("#" + prevView.params.id).css('display', 'block');
        };

        /**
         * update the settings/icons of current view loaded into the wizard
         *
         * @param {jQuery}: element the DOM element of the widget
         * @param {json}: params the settings to be updated for loaded view
         * @param  {json}: actionBttnsStatus the status of action buttons for the loaded view
         *
         */

        _self.doUpdateView = function (dom, params, actionBttnsStatus) {

            $(dom).find(".active-step .icon-wrap").removeClass("active-step-direction");
            $(dom).find(".soe-icon-"+params.vname).removeClass("done");
            $(dom).find(".soe-step-desc-"+params.vname).removeClass("done");
            $(dom).find(".sbcOfflineEngineering-steps  .active-step").removeClass("active-step");
            $(dom).find(".soe-icon-"+params.id).addClass("active-step");
            //Note: append active-step-direction class contents only one, not for all '.active-step' classes
            $(dom).find(".active-step  .icon-wrap").addClass("active-step-direction");
            $(dom).find(".soe-step-desc-"+params.id).addClass("active-step");

            //remove .done status for next steps if are coming back
            $.each($(".sbcOfflineEngineering-steps div[class^='soe-icon'].active-step").nextAll(), function (index, element) {
                if($(element).hasClass("done")) {
                    $(element).removeClass("done");
                    $(element).children("div[class^='soe-step-desc']").removeClass("done");
                }
            });

            //add .done status for previous steps if are moving forward
            $.each($(".sbcOfflineEngineering-steps div[class^='soe-icon'].active-step").prevAll(), function (index, element) {
                if(!$(element).hasClass("done")) {
                    $(element).addClass("done");
                    $(element).children("div[class^='soe-step-desc']").addClass("done");
                }
            });

            if(params.backBttnMsg) {
                $(dom).find("#sbcOfflineEngineering-backBttn").val(params.backBttnMsg)
                    .css('display', 'inline-block').prop("disabled", false);
            } else {
                $(dom).find("#sbcOfflineEngineering-backBttn").css('display', 'none');
            }
            if(params.nextBttnMsg) {
                $(dom).find("#sbcOfflineEngineering-nextBttn").val(params.nextBttnMsg)
                    .css('display', 'inline-block').prop("disabled", false);
            } else {
                $(dom).find("#sbcOfflineEngineering-nextBttn").css('display', 'none')
            }

            if(params.cancelBttnMsg) {
                $(dom).find("#sbcOfflineEngineering-cancelBttn").val(params.cancelBttnMsg)
                    .css('display', 'inline-block').prop("disabled", false);
            } else {
                $(dom).find("#sbcOfflineEngineering-cancelBttn").css('display', 'none');
            }

            if (actionBttnsStatus) {
                if (actionBttnsStatus.disabledBack != undefined)
                    $(dom).find("#sbcOfflineEngineering-backBttn").prop("disabled", actionBttnsStatus.disabledBack);
                if (actionBttnsStatus.disabledNext != undefined)
                    $(dom).find("#sbcOfflineEngineering-nextBttn").prop("disabled", actionBttnsStatus.disabledNext);
                if (actionBttnsStatus.disabledCancel != undefined)
                    $(dom).find("#sbcOfflineEngineering-cancelBttn").prop("disabled", actionBttnsStatus.disabledCancel);
            }

            //hide them all.
            var contentWrapper = $(dom).find("#sbcOfflineEngineering-contentWrapper");
            contentWrapper.children().css("display", "none");
        };

        /**
         * cancel running operation within loaded view
         */

        _self.cancelOp = function() {
            if(_self.currentView.cancel)
                _self.currentView.cancel(_self.component);
        };

        /**
         * Gets the view(prev/next) to be loaded into the wizard
         *
         * @param  currentView: the current view loaded into the widget
         * @param  type: the view type "next" or "prev"
         *
         * @returns {view} return view to be loaded based on 'type' argument
         */

        _self.getUIView = function (currentView, type) {
            var viewIndex = utils.getArrElementIndex(_self.wizardOrder, currentView.params.vname);
            var view,
                selectedModules,
                scannedModules,
                availableModules;

            if(type == "next") {
                if(viewIndex >= _self.wizardOrder.length - 1) {
                    view = _self.wizardOrder[0];
                } else {
                    view = _self.wizardOrder[viewIndex+1];
                }
            }  else if(type == "prev") {
                if(viewIndex > 0) {
                    view = _self.wizardOrder[viewIndex-1];
                } else {
                    view = _self.wizardOrder[0];
                }
            } else {
              //return current view
               view = currentView;
            }

            selectedModules  = _self.soeData.selectedModules || {};
            scannedModules = _self.soeData.scannedModules || {};
            availableModules = {datapoints:selectedModules.datapoints && scannedModules.datapoints,
                alarms:selectedModules.alarms && scannedModules.alarms,
                hdlogs:selectedModules.hdlogs && scannedModules.hdlogs};

            if((!availableModules.datapoints && view == datapointsView)
                || (!availableModules.alarms && view == alarmlistView)
                || (!availableModules.hdlogs && view == hdlogView)) {
                view =  _self.getUIView(view, type);
            }
            return view;
        };

        //TODO: update wizardOrder array whenever new views are added
        _self.wizardOrder = [
            selectView,
            scanView,
            datapointsView,
            alarmlistView,
            hdlogView,
            importView,
            finishView
        ];

    };

    //extend and set up prototype chain
    SbcOfflineEngineeringWizard.prototype = Object.create(Widget.prototype);
    SbcOfflineEngineeringWizard.prototype.constructor = SbcOfflineEngineeringWizard;

    var lex = lexicons[0];


    /**
     * Do initial setup of the DOM for the widget. This will set up the DOM's
     * structure.
     *
     * @param {jQuery} element the DOM element into which to load this widget
     */
    SbcOfflineEngineeringWizard.prototype.doInitialize = function (dom) {
        var _self = this;
        _self.soeData = {};
        $(dom).addClass("sbcOfflineEngineering-wizard-parent");
        $(dom).parent().addClass("sbcOfflineEngineering-background");
        $(dom).closest(".hx").addClass("sbc-root");
        $(dom).closest(".niagara-env-wb").addClass("sbc-root");

        $(dom).html(template({
            title:lex.get("sbc.offlineEngineering.title"),
            backBttnMsg:lex.get("sbc.offlineEngineering.back"),
            nextBttnMsg:lex.get("sbc.offlineEngineering.next"),
            cancelBttnMsg:lex.get("sbc.offlineEngineering.cancel"),
            select:lex.get("sbc.offlineEngineering.select"),
            scan:lex.get("sbc.offlineEngineering.scan"),
            datapoints:lex.get("sbc.offlineEngineering.datapoints"),
            alarms:lex.get("sbc.offlineEngineering.alarms"),
            hdlogs:lex.get("sbc.offlineEngineering.hdlogs"),
            import:lex.get("sbc.offlineEngineering.import"),
            finish:lex.get("sbc.offlineEngineering.finish"),
            one:lex.get("sbc.offlineEngineering.select.one"),
            two:lex.get("sbc.offlineEngineering.scan.two"),
            three:lex.get("sbc.offlineEngineering.datapoints.three"),
            four:lex.get("sbc.offlineEngineering.alarms.four"),
            five:lex.get("sbc.offlineEngineering.hdlogs.five"),
            six:lex.get("sbc.offlineEngineering.import.six"),
            seven:lex.get("sbc.offlineEngineering.finish.seven")

        }));

        $(dom).find("#sbcOfflineEngineering-backBttn").click(_self.prevView);
        $(dom).find("#sbcOfflineEngineering-nextBttn").click(_self.nextView);
        $(dom).find("#sbcOfflineEngineering-cancelBttn").click(_self.cancelOp);

    };

    /**
     * Loads offline engineering wizard
     *
     * @param component: the complex value
     */
    SbcOfflineEngineeringWizard.prototype.doLoad = function (component) {

        try {
            var _self = this;
            _self.component = component;
            _self.loadNextView(_self.wizardOrder[0]);

        }catch (e) {
            console.log("error loading the initial view");

        }

    };

    /**
     * Describe what kind of data you can read out of this widget.
     *
     * @returns {Promise} promise to be resolved with the current value
     */
    SbcOfflineEngineeringWizard.prototype.doRead = function () {
        //promises are optional - the slot could also be returned directly
        //return Promise.resolve(
        /** send value to be read from the wizard **/
        //);
    };

    /**
     * cleanup offline engineering wizard
     *
     * @param component: the complex value
     */
    SbcOfflineEngineeringWizard.prototype.doDestroy = function (params) {

        try {
            var _self = this;
            var dom = _self.jq();
            $(dom).removeClass("sbcOfflineEngineering-wizard-parent");
            $(dom).parent().removeClass("sbcOfflineEngineering-background");
            $(dom).closest(".sbc-root").removeClass("sbc-root");
        }catch (e) {
            console.log("error loading the initial view");

        }

    };

    return SbcOfflineEngineeringWizard;
});