/**
 * * Copyright (c) 2018. Honeywell International, Inc. All Rights Reserved.
 */

define(['baja!',
        'jquery',
        'Promise',
        'underscore'],
    function (Baja,
              $,
              Promise,
              _) {

        'use strict';


        var nscroll = function (options) {

            var _self = $(this);

            var TWS = {
                    $origContent:null,
                    front:0,
                    rear:0,
                    $content:null,
                    total:-1
                },
                $wrapper = $('<div class="scroll-wrapper"></div>'),
                $parent,
                ratio,
                _rowHeight = 32,
                _verticalScroll=-1,
                _options = {
                    getContent:null,
                    onLoad: null,
                    onUnload: null,
                    updateLimit: 25,
                    loadLimit:10,
                    limitedVisibility:false,
                    visibleLimit:100,
                    parent:null,
                    contentSelector:null,
                    contentSize:-1
                };

            var _unloadContent = function (loadAfter) {
                if(loadAfter === true) {
                    TWS.$content.children().slice(-(_options.updateLimit)).remove();
                    TWS.rear -= _options.updateLimit;
                } else if(loadAfter === false) {
                    TWS.$content.children().slice(0, _options.updateLimit).remove();
                    TWS.front += _options.updateLimit;
                }
                if(_options.onUnload) {
                    _options.onLoad.call();
                }
            };

            var _loadContent = function(content, loadAfter) {
                $wrapper.remove('.loading');
                var $tcontent = $(content).filter(function () {
                    return !(this.nodeType === 3);
                });

                if(loadAfter === true) {
                    TWS.$content.append(content);
                    //$(_options.contentSelector).append($tcontent);
                    TWS.rear += parseInt($tcontent.length);
                } else if(loadAfter === false && (TWS.front - $tcontent.length) >= 0) {
                    TWS.$content.prepend($tcontent);
                    TWS.front -= parseInt($tcontent.length);
                }


                if(_options.limitedVisibility && _options.visibleLimit < TWS.$content.children().length ) {
                    //remove some children before or after - on opposite end
                    _unloadContent(!loadAfter);
                }
                if(_options.onLoad) {
                    _options.onLoad.call(null, $tcontent);
                }
            };

            var _loadContentOnly = function(content, loadAfter) {
                $wrapper.remove('.loading');
                var $tcontent = $(content).filter(function () {
                    return !(this.nodeType === 3);
                });

                if(loadAfter === true) {
                    TWS.$content.append(content);
                    //$(_options.contentSelector).append($tcontent);
                    TWS.rear += parseInt($tcontent.length);
                }

                if(_options.onLoad) {
                    _options.onLoad.call();
                }
            };


            var _updateDataAndView = function (loadAfter) {
                var content = null, tlimit;
                if(loadAfter === true) {
                    if(TWS.$origContent.children().length > (TWS.rear+1)) {
                        tlimit = (TWS.$origContent.children().length - TWS.rear > _options.visibleLimit)
                            ? _options.visibleLimit : parseInt(TWS.$origContent.children().length - TWS.rear);
                        content = TWS.$origContent.children().slice(TWS.rear, tlimit);
                    } else {
                        content = _options.getContent();
                        TWS.$origContent.append(content);
                        content = $(content);
                    }
                } else if(loadAfter === false) {
                    if(TWS.front > 0) {
                        tlimit = (TWS.front > _options.loadLimit)
                            ? (TWS.front - _options.loadLimit) : 0;
                        content = TWS.$origContent.children().slice(tlimit, TWS.front);
                    }
                }
                if(content != null && content.length > 0) {
                    _loadContent((content), loadAfter);
                }
            };

            var _replaceDataAndView = function (start, end) {
                var content = null, tlimit;
                content = _options.getContent(start, end);
                var $tcontent = $(content).filter(function () {
                    return !(this.nodeType === 3);
                });

                if(content != null && content.length > 0) {
                    //TODO: is this required?
                    TWS.$origContent.html(content);
                    TWS.$content.html($tcontent);
                    TWS.front = 0;
                    TWS.rear += parseInt($tcontent.length);
                    if(_options.onLoad) {
                        _options.onLoad.call(null, $tcontent);
                    }
                }
            };


            var _monitor = function (event) {
                // console.log("_monitor: "  + event);
                var target = event.target;


                //replace content only if target element is scrolled and its scrolled vertically!!
                if($(target).is(TWS.$content) && (_verticalScroll != target.scrollTop)) {
                    var isEnd = target.scrollHeight - target.scrollTop === target.clientHeight;
                    var scrollTop = target.scrollTop;
                    var start, end, pos, r;
                    _rowHeight = $(TWS.$content.children(":first")).outerHeight();
                    if(ratio) {
                        // pos = (scrollTop/_rowHeight);
                        // r =  pos * ratio;
                        // start = Math.round(r);
                        //
                        // //special condition - if user scrolls to end of page, detect end of page scroll and display all remaining rows.
                        // if(scrollTop + target.clientHeight >= target.scrollHeight) {
                        //     end = _options.contentSize;
                        // } else {
                        //     end = Math.round(start + (_options.visibleLimit));
                        // }

                        /* Display only datapoints as configured by visibleLimit, if the contentsize (total datapoints) is more than visibleLimit then
                         * divide the available screen size into equal parts (contentSize/visibleLimit) and display the datapoints based on scroll position.
                         * scrollTop can never exceed (scrollHeight minus clientHeight), divide based on scrollTop positions and not scrollHeight.
                         */
                        var maxScroll = target.scrollHeight - target.clientHeight;
                        var scrollRange = maxScroll/ratio;
                        var currentPos = Math.floor(scrollTop/scrollRange);
                        start = (currentPos * _options.visibleLimit) + 1;
                        end = Math.round(start + (_options.visibleLimit));

                        if(start < 0) start = 0;
                        if(end > _options.contentSize) {
                            end = _options.contentSize;
                            if((end - start) < _options.visibleLimit) {
                                start = (end - _options.visibleLimit);
                            }
                        }

                        _replaceDataAndView(start, end);
                        //TODO: load progress icon
                    }
                    _verticalScroll = scrollTop;
                }
            };

            //TODO: call these on keypress (up, down, page up, page down)

            var _observe = function (event) {
                // console.log("in Observe!! " + event);
                var target = event.target;
                //var isEnd = target.scrollHeight - target.scrollTop === target.clientHeight;
                if((target.scrollHeight - target.scrollTop) < (target.clientHeight + 60)) {
                    _updateDataAndView(true);
                } else if(target.scrollTop < 120) {
                    _updateDataAndView(false);
                }

            }



            var _reset = function () {
                TWS = {
                    $origContent: null,
                    front: 0,
                    rear: 0,
                    $content: null,
                },
                    $wrapper = $('<div class="scroll-wrapper"></div>'),
                    _options = {
                        getContent: null,
                        onLoad: null,
                        onUnload: null,
                        updateLimit: 25,
                        loadLimit: 10,
                        limitedVisibility: false,
                        visibleLimit: 100,
                        parent: null,
                        contentSelector: null
                    };
            };

            var _initialize = function () {
                _reset();
                $.extend(true, _options, options);
                $parent = _options.parent || $("body");
                if(_options.content == null) return new Error("No contents to display!");
                if(_options.contentSize > _options.visibleLimit) {
                    ratio = _options.contentSize/_options.visibleLimit;
                    TWS.total = _options.contentSize;
                }
                $wrapper.append(_options.content);
                TWS.$origContent = $("<div></div>").html(_options.content);
                TWS.$origContent = $(TWS.$origContent).find(_options.contentSelector);
                $parent.append($wrapper);
                TWS.$content = $parent.find(_options.contentSelector);
                TWS.rear = parseInt(TWS.$content.children().length);

                //$wrapper.scroll(_monitor);
                $wrapper.find(_options.contentSelector).keyup(_observe);
//                $wrapper.find(_options.contentSelector).scroll();

                $wrapper.find(_options.contentSelector).mouseup(_monitor);
            };

            _initialize();

            return {
                destroy: function () {
                    //TODO: clear the widget data
                },
                //Note: This will only load contents at the end without unloading anything if exceeds limit
                //Note: Very much like infinite scrolling.
                loadContent : function () {
                    var content = null, tlimit;
                    if(TWS.$origContent.children().length > (TWS.rear+1)) {
                        tlimit = (TWS.$origContent.children().length - TWS.rear > _options.visibleLimit)
                            ? _options.visibleLimit : parseInt(TWS.$origContent.children().length - TWS.rear);
                        content = TWS.$origContent.children().slice(TWS.rear, tlimit);
                    } else {
                        content = _options.getContent();
                        TWS.$origContent.append(content);
                        content = $(content);
                    }
                    if(content != null && content.length > 0) {
                        _loadContentOnly((content), true);
                    }
                },
                updateOptions : function (options) {
                    $.extend(true, _options, options);
                    if(_options.contentSize > _options.visibleLimit) {
                        ratio = _options.contentSize/_options.visibleLimit;
                        TWS.total = _options.contentSize;
                    }
                }
            };
        };

        return nscroll;

    });
