/**
 * * Copyright (c) 2018. Honeywell International, Inc. All Rights Reserved.
 */


define(['baja!',
        'jquery',
        'Promise',
        'underscore',
        'dialogs',
        'nmodule/galileoWidgets/rc/js/GalileoUITreeView',
        'nmodule/galileoWidgets/rc/js/GalileoUITable',
        'hbs!nmodule/sbcOfflineEngineering/rc/hbs/datapointsDisplayTable',
        'hbs!nmodule/sbcOfflineEngineering/rc/hbs/datapointsDisplayTable-Row',
        'nmodule/webEditors/rc/fe/fe',
        'nmodule/webEditors/rc/fe/baja/FrozenEnumEditor',
        'nmodule/sbcOfflineEngineering/rc/js/util/utils',
        'hbs!nmodule/sbcOfflineEngineering/rc/hbs/datapointsView',
        'hbs!nmodule/sbcOfflineEngineering/rc/hbs/dpFilterOverlay',
        'nmodule/sbcOfflineEngineering/rc/js/nscroll',
        'nmodule/sbcOfflineEngineering/rc/js/util/SearchFilter',
        'lex!sbcOfflineEngineering',
        'nmodule/sbcOfflineEngineering/rc/js/lib/sorttable',
        'nmodule/sbcOfflineEngineering/rc/js/lib/jquery-ui',
        'baja!sbc:SbcRegisterTypeEnum,sbc:SbcCounterTypeEnum,sbc:SbcBinaryTypeEnum,sbc:SbcRegisterFloatTypeEnum,sbc:SbcStringTypeEnum'
    ],
    function (baja,
              $,
              Promise,
              _,
              dialogs,
              treeView,
              uiTable,
              stable,
              stableRow,
              fe,
              enumEditor,
              utils,
              template,
              filterTemplate,
              nscroll,
              SearchFilter,
              Lexicons) {

        'use strict';

        /**
         * Description of your widget.
         *
         * @class
         * @alias module:nmodule/trendMigration/rc/js/migration/MigrationSelectView
         */
        var datapointsView = new function(){

            //define local variables and functions
            var $element = null, $widgetContainer = null, sbcService = null, subscriber;
            var selectedModules = {}, scannedList = {} ;
            var _resolveNext = null;

            var lex = Lexicons[0];

            var treeDeviceView = null;
            var datapointsPath = "file:^offline/engineering/system.bog|bog:|slot:|";
            var _deviceMap = {};
            var _deviceHrchy = {};
            var selectedDatapointsList = {};
            var selectedDeviceDatapoints = [], _selectedDeviceDpFullList = [];
            var overwriteDialog = null;
            var devicePointsData = {data:[]};
            var _prepImportDialog = null;
            var _configUpdateLimit = 200;
            var delimiter = "***"
            var selectedDevices;

            var tableLimits = {
                visibleLimit:100,
                updateLimit:50
            }

            var displayTableCurrentPointer = tableLimits.updateLimit;
            var displayTableScroll;

            var selectTableCurrentPointer = tableLimits.updateLimit, selectTableScroll;

            var _filterDpDialog = null;
            var _savedDpDocData = {};
            var _pointNameMap = {
                "R" : "1",
                "R FLOAT":"1",
                "F":"2",
                "F VOL":"2",
                "C":"3",
                "I":"4",
                "O":"4",
                "T":"5",
                "DB":"6",
                "DB RAM":"6",
                "TEXT":"7",
                "TEXT RAM":"7"
            };
            var typingTimeout = null;

            /*var configTimer = null;

            var monitorConfigurationsTimer = function(resolveNext){
                configTimer = setInterval(
                    function(){
                        if(!sbcService.getDataPoints().getDataPointsConfigureState()){
                            _resolveNext = resolveNext;
                             window.clearInterval(configTimer);
                        }
                    },1000
                );
            };*/

            //return true if datapoints are available on client-side for given device
            var _hasDatapointsData = function (key) {
                return (_deviceMap[key]) ? true :  false;
            };

            /**
             * return saved information for given key (device) in _deviceMap
             * @param key
             * @returns {{}}
             * @private
             */
            var _getSelectedDeviceInfo = function (key) {
                var tokens = key.split(".");
                var dlevel = 0;
                var ddata = _deviceMap;
                while(dlevel < tokens.length) {
                    ddata = ddata[tokens[dlevel]];
                    dlevel++;
                }
                return ddata;
            };

            /**
             * For given input device (or any descandant) it will return all datapoints part of the device or
             * selected input in device hierarchy
             * @param datapoints
             * @param datapointsDetail
             * @returns {*}
             */
            var getSelectedDeviceDatapoints = function(datapoints, datapointsDetail) {
                var i,j;
                for( i in datapointsDetail) {
                    if(i === "datapoints") {
                        for(j = 0; j < datapointsDetail[i].length; j++) {
                            datapoints.push(datapointsDetail[i][j]);
                        }
                    }
                    else {
                        getSelectedDeviceDatapoints(datapoints,datapointsDetail[i]);
                    }
                }
                return datapoints;
            };

            var _setDataFormat = function(enumType, elementClass) {
                return fe.buildFor({
                    value: baja.$(enumType),
                    dom: $('#soe-datapoints-display-table').find('.'+elementClass),
                    type: enumEditor
                });
            };

            var _getClass = function(mediaType) {
                if(mediaType === 'R') {
                    return 'register';
                } else if(mediaType === 'R FLOAT') {
                    return 'registerFloat';
                } else if(mediaType === 'C' || mediaType === 'T') {
                    return 'counter';
                } else if(mediaType === 'F' || mediaType === 'F VOL' ||
                    mediaType === 'O' || mediaType === 'I') {
                    return 'boolean';
                } else if(mediaType === 'TEXT' || mediaType === 'TEXT RAM') {
                    return 'string';
                }
                return "";
            };



            //checkbox handler to add selected datapoint to selected items table
            var _addDeleteSelectedDatapointsList = function (event, checkbox) {
                var _checkbox = checkbox || this;
                var val = $(_checkbox).val();
                var dataFormatIndex = $(_checkbox.parentElement.parentElement).find('option:selected').val();
                if(dataFormatIndex == undefined) {
                    dataFormatIndex = "-1";
                }
                var tokenizer, selectedListDisplayLength, selectedListLength;

                var parentTR = $(_checkbox).closest("tr");
                var mediaType, mediaTypeIndex = $("#soe-datapoints-display-table thead th[data-field='dataType']").index();
                var memoryAddress, memoryAddressIndex = $("#soe-datapoints-display-table thead th[data-field='memoryAddress']").index();
                var comments, commentsIndex =  $("#soe-datapoints-display-table thead th[data-field='comments']").index();


                if($(_checkbox).is(":checked")) {
                    if(selectedDatapointsList[val]) {
                        //already added - update info?
                    } else {
                        mediaType = parentTR.find("td:nth-child("+ (mediaTypeIndex+1) +")").text();
                        memoryAddress = parentTR.find("td:nth-child("+ (memoryAddressIndex+1) +")").text();
                        comments = parentTR.find("td:nth-child("+ (commentsIndex+1) +")").attr("title");

                        tokenizer = val.split(delimiter);
                        selectedDatapointsList[val] = {
                            device:tokenizer[0],
                            pointName:tokenizer[1],
                            selected:1,
                            dataFormat:dataFormatIndex,
                            mediaType:mediaType,
                            memoryAddress:memoryAddress,
                            comments:comments
                        };

                        selectedListLength = Object.keys(selectedDatapointsList).length;
                        selectTableScroll.updateOptions({
                            contentSize:selectedListLength
                        });
                    }
                } else {
                    $("#soe-datapoints-selected-table").find("tr[id='"+utils.escapeJQID(val)+"']").remove();
                    delete selectedDatapointsList[val];
                    selectedListLength = Object.keys(selectedDatapointsList).length;

                    selectTableScroll.updateOptions({
                        contentSize:selectedListLength
                    });
                    selectTableCurrentPointer--;
                }

                selectedListDisplayLength = $element.find("#soe-datapoints-selected-table > tbody").children().length;

                if((selectedListDisplayLength < selectedListLength) && selectedListDisplayLength < tableLimits.visibleLimit) {
                    selectTableScroll.loadContent();
                }


            };


            function _selectAllDatapointsForDevice(event, _selectColumn) {
                var isChecked = $(_selectColumn).children("input[type='checkbox']").is(":checked");
                var selectColIndex = $(_selectColumn).index();
                var allDatapoints, i, val, tokenizer, dataFormatIndex, dp, dpRow;

                var $tableElement = $element.find("#soe-datapoints-selected-table > tbody:last-child");

                if(isChecked) {
                    allDatapoints = $("#soe-datapoints-display-table")
                        .find("tr:not(.displayNone) td:nth-child("+ (selectColIndex+1) +") input[type='checkbox']");

                    var parentTR;
                    var mediaType, mediaTypeIndex = $("#soe-datapoints-display-table thead th[data-field='dataType']").index();
                    var memoryAddress, memoryAddressIndex = $("#soe-datapoints-display-table thead th[data-field='memoryAddress']").index();
                     var comments, commentsIndex =  $("#soe-datapoints-display-table thead th[data-field='comments']").index();

                    for(i=0; i< allDatapoints.length; i++) {
                        val = $(allDatapoints[i]).val();
                        if(!selectedDatapointsList[val]) {
                            parentTR = $(allDatapoints[i]).closest("tr");

                            mediaType = parentTR.find("td:nth-child("+ (mediaTypeIndex+1) +")").text();
                            memoryAddress = parentTR.find("td:nth-child("+ (memoryAddressIndex+1) +")").text();
                            comments = parentTR.find("td:nth-child("+ (commentsIndex+1) +")").attr("title");

                            //reset
                            dataFormatIndex = -1;
                            tokenizer = val.split(delimiter);
                            dataFormatIndex = $(allDatapoints[i].parentElement.parentElement).find('option:selected').val();
                            if(dataFormatIndex == undefined) {
                                dataFormatIndex = "-1";
                            }
                            selectedDatapointsList[val] = {
                                device:tokenizer[0],
                                pointName:tokenizer[1],
                                selected:1,
                                dataFormat:dataFormatIndex,
                                mediaType:mediaType,
                                memoryAddress:memoryAddress,
                                comments:comments
                            };
                        }
                    }

                    if(allDatapoints.length <  selectedDeviceDatapoints.length) {
                        for(i=0; i< selectedDeviceDatapoints.length; i++) {
                            dp = selectedDeviceDatapoints[i];
                            val = dp.select;
                            if(!selectedDatapointsList[val]) {
                                //reset
                                selectedDatapointsList[val] = {
                                    device:dp.deviceName,
                                    pointName:dp.pointName,
                                    selected:1,
                                    dataFormat: ((_getClass(dp.dataType) == "") ? "-1" : "1"),
                                    memoryAddress:dp.memoryAddress,
                                    mediaType:dp.dataType,
                                    comments:dp.comments
                                };
                            }
                        }
                    }
                } else {
                    for(i=0; i< selectedDeviceDatapoints.length; i++) {
                        dp = selectedDeviceDatapoints[i];
                        val = dp.select;
                        if($tableElement.find("tr[id='"+utils.escapeJQID(val)+"']").length > 0) {
                            $tableElement.find("tr[id='"+utils.escapeJQID(val)+"']").remove();
                            selectTableCurrentPointer--;
                        }
                        delete selectedDatapointsList[val];
                    }
                }

                //update selectedDatapoints count
                var selectedDPCount = Object.keys(selectedDatapointsList).length;

                //update selected points count
                $element.find("#soe-datapoints-selected-label-count").text(selectedDPCount);
                $element.find("#soe-datapoints-selected-label-count").removeClass("spinner");


                selectTableScroll.updateOptions({
                    contentSize:selectedDPCount
                });

                if($tableElement.children().length < tableLimits.visibleLimit) {
                    selectTableScroll.loadContent();
                }
            }

            var _selectAllDatapointsHeaderForDevice = function () {
                //restore previously selected elements
                var i, val, isAllDatapointsSelected  = true;
                if(selectedDeviceDatapoints.length > 0) {
                    for(i=0; i < selectedDeviceDatapoints.length;i++) {
                        val = selectedDeviceDatapoints[i].select;
                        if(!selectedDatapointsList[val]) {
                            isAllDatapointsSelected = false;
                            break;
                        }
                    }
                }
                $element.find("#soe-datapoints-display-table th input[type='checkbox']").prop("checked", isAllDatapointsSelected);
            };

            var _updateDisplaySelectedDeviceDatapoints = function (newRows) {
                if(newRows == undefined) newRows = [];
                //Adding/updating the data conversion type for the points
                Promise.all([_setDataFormat("sbc:SbcRegisterTypeEnum", 'register'), _setDataFormat("sbc:SbcCounterTypeEnum", 'counter'),
                    _setDataFormat("sbc:SbcRegisterFloatTypeEnum", 'registerFloat'), _setDataFormat("sbc:SbcBinaryTypeEnum", 'boolean'),
                    _setDataFormat("sbc:SbcStringTypeEnum", 'string')]).then(function(){
                    //Adding listener on changing the data conversion type and updating the selected point
                    $("#datapointsView .gcs_table").find("select").on("change", function(event) {
                        var checkbox = $(event.currentTarget.parentElement.parentElement).find('input[type=checkbox]');
                        if(checkbox.prop('checked')) {
                            var point = selectedDatapointsList[checkbox.val()];
                            point.dataFormat = $(event.currentTarget).val();
                        }
                    });
                    //Updating the data conversion type for the points selected
                    if(selectedDeviceDatapoints.length > 0) {
                        var devName = selectedDeviceDatapoints[0].deviceName;
                        // var tr = $element.find("#soe-datapoints-selected-table tr[id^='"+utils.escapeJQID(devName)+"']");
                        $.each(newRows, function(index, element) {
                            var val = $(element).find("td input[type='checkbox']").val();
                            var point = selectedDatapointsList[val];
                            if(point) {
                                $(element).find("select").val(point.dataFormat);
                            }
                        });
                    }
                });


                //restore previously selected elements
                if(selectedDeviceDatapoints.length > 0) {
                    var deviceName = selectedDeviceDatapoints[0].deviceName;

                    var t = $element.find("#soe-datapoints-display-table td input[type='checkbox']");
                    $.each(t, function(index, element) {
                        var val = $(element).val();
                        if(selectedDatapointsList[val]) {
                            $(element).prop("checked", true);
                        }
                    });
                }

                //check table header if all the datapoints for the device are already selected.
                _selectAllDatapointsHeaderForDevice();

                //add the checkbox selection handler
                $("#datapointsView .gcs_table").find("td input[type='checkbox']").on("click", function(event) {
                    var _checkbox = this;
                    window.setTimeout(function(event, _checkbox) {
                        _addDeleteSelectedDatapointsList(event, _checkbox);
                        if($(_checkbox).prop("checked")) {
                            _selectAllDatapointsHeaderForDevice();
                        } else {
                            $element.find("#soe-datapoints-display-table th input[type='checkbox']").prop("checked", false);
                        }
                        //update selected points count
                        $element.find("#soe-datapoints-selected-label-count").text(Object.keys(selectedDatapointsList).length);
                    }, 0, event, _checkbox);
                });
            };



            var _getTRContent = function (start, end) {
                var l1 = (start  != undefined) ? start : displayTableCurrentPointer,
                    l2 = (end != undefined) ? end : (displayTableCurrentPointer+tableLimits.updateLimit);
                displayTableCurrentPointer = l2;

                var e = stableRow({
                    columns: _deviceMap.columns,
                    rows: selectedDeviceDatapoints.slice(l1, l2)
                });

                return e.trim();
            };

            var _getSelectTableTRContent = function (start, end) {
                var i, tokenizer,
                    l1 = (start  != undefined) ? start : selectTableCurrentPointer,
                    l2 = (end != undefined) ? end : (selectTableCurrentPointer+tableLimits.updateLimit),
                    keys = Object.keys(selectedDatapointsList),
                    tbl_TD = "</td><td class='soe-delete-selected-icon' title='"+
                    lex.get("sbc.offlineEngineering.datapoints.delete")+"'></td></tr>",
                    buffer = "";

                if(keys.length > 0) {
                    l2 = l2 < keys.length ? l2 : keys.length;

                    for(i=l1; i < l2; i++) {
                        var val = keys[i];
                        tokenizer = val.split(delimiter);
                        buffer += "<tr class='ux-table-row' id='"+val+"'><td>"+tokenizer[1]+"</td><td title='"+tokenizer[0]+"'>"
                        +tokenizer[0]+tbl_TD;
                    }
                    selectTableCurrentPointer = l2;
                }
                return buffer;
            }

            var _displaySelectedDeviceDatapoints = function () {

                //remove previously created table... at any point only one device points should be displayed.
                $element.find("#soe-datapoints-display-content").empty();

                var ahtml = stable({
                    columns: _deviceMap.columns, rows: selectedDeviceDatapoints.slice(0, tableLimits.visibleLimit),
                    options: {altStripes: false, tableId: 'soe-datapoints-display-table'}
                });
                displayTableCurrentPointer = tableLimits.updateLimit;

                displayTableScroll = nscroll({ getContent:_getTRContent,
                    onLoad: _updateDisplaySelectedDeviceDatapoints,
                    onUnload: null,
                    updateLimit: 50,
                    loadLimit:tableLimits.updateLimit,
                    limitedVisibility:true,
                    visibleLimit:tableLimits.visibleLimit,
                    parent:$element.find("#soe-datapoints-display-content"),
                    contentSelector:'#soe-datapoints-display-table  tbody',
                    content:ahtml,
                    contentSize:selectedDeviceDatapoints.length
                });

                $("#soe-datapoints-display-table  tbody").scroll(function (e) {
                    var _self = this;
                    var left = $(_self).scrollLeft();
                    $("#soe-datapoints-display-table .ux-table-head").scrollLeft($(_self).scrollLeft());
                });


                //wrap up - adding rows is done!
                //Display total # of points (count) for selected device
                $element.find("#soe-datapoints-displayed-label-count").text(selectedDeviceDatapoints.length);

                var $displayTable = $element.find("#soe-datapoints-display-table");

                //Note: hack to avoid delay in selection
                $("#soe-datapoints-display-table th input[type='checkbox']")[0].addEventListener("click", function(event) {
                    var _selectInput = this;
                    var isChecked = $(_selectInput).is(":checked");
                    var selectColIndex = $(_selectInput).parent().index();
                    var allDatapoints = $("#soe-datapoints-display-table")
                        .find(" td:nth-child("+ (selectColIndex+1) +") input[type='checkbox']");
                    allDatapoints.prop("checked", isChecked);
                });
                
                //restore previously selected datapoints and update data format selection, attach listeners
                _updateDisplaySelectedDeviceDatapoints($("#soe-datapoints-display-table > tbody > tr"));

                //handler to enable select all datapoints with the datapoints display table
                $displayTable.find("th[data-field-type='checkbox']")[0].addEventListener("click", function(event) {
                    var _selectColumn = this;
                    $element.find("#soe-datapoints-selected-label-count").addClass("spinner");
                    window.setTimeout(_selectAllDatapointsForDevice, 0, event, _selectColumn);
                });

                $element.find(".treeView .spinner").removeClass("spinner");

                //enable temporarily disabled selection while loading...
                $element.find("#soe-datapoints-display-table input[type='checkbox']").prop("disabled", false);
                $element.find(".sbcOfflineEngineering-actionButtons input").prop("disabled", false);

            };

            var _searchByDpName = function() {
                typingTimeout = null;
                var searchtext = $("#soe-datapoints-searchBox > input").val(), srchFltr ;
                srchFltr = new SearchFilter(searchtext, true);
                selectedDeviceDatapoints = srchFltr.filter(_selectedDeviceDpFullList, "pointName");
                _displaySelectedDeviceDatapoints();
            };

            /**
             * _displayDatapointsInfo: read (from js object) and display datapoints for selected device in hierarchy
             * @param datapointsDetail
             */
            var _displayDatapointsInfo = function (datapointsDetail, currentTarget) {

                //get all datapoints in the hierarchy of selected device
                selectedDeviceDatapoints = selectedDeviceDatapoints.concat(getSelectedDeviceDatapoints([], datapointsDetail))
                selectedDeviceDatapoints = utils.uniqList(selectedDeviceDatapoints, JSON.stringify);
                _selectedDeviceDpFullList = selectedDeviceDatapoints;
                if ($("#soe-datapoints-searchBox > input").val() != "") {
                    _searchByDpName();
                } else {
                    _displaySelectedDeviceDatapoints();
                }

            };

            var _createAndDisplayDpInfo = function (data, currentTarget, fname) {
                var docData = null, device,deviceName, flags = "", ftexts = [];
                var filterTagTexts = [];
                if (null == data) {
                    docData =_savedDpDocData[fname];
                } else {
                    docData = data;
                }
                 device = docData.find("Device");
                 deviceName = $(device).attr("name");
                //column order point name --> media type --> address --> data format --> comment
                _deviceMap.columns = [
                    {name:"", type:"checkbox", field:"select"},
                    {name:lex.get("sbc.offlineEngineering.datapoints.pointName"), field:"pointName"},
                    {name:lex.get("sbc.offlineEngineering.datapoints.dataType"), field:"dataType"},
                    {name:lex.get("sbc.offlineEngineering.datapoints.memoryAddress"), field:"memoryAddress"},
                    {name:lex.get("sbc.offlineEngineering.datapoints.dataFormat"), field:"dataFormat"},
                    {name:lex.get("sbc.offlineEngineering.datapoints.comments"), field:"comments"}
                ];
                _deviceMap[deviceName] = {};
                _deviceHrchy[deviceName] = {name:deviceName, path:deviceName, children: []};

                if (fname){
                    $(".pointInFilter").each(function() {
                        if(this.checked) {
                            flags+= this.id.charAt(this.id.length-1) + ",";
                        }
                    });
                    flags = flags.substring(0,flags.length-1);
                    ftexts = $(".dpTag");
                    filterTagTexts = $(".dpTrueTags");
                } else {
                    flags = sbcService.getSelectedMediaTypes() == "" ? sbcService.getSelectedDefaultMediaTypes():sbcService.getSelectedMediaTypes();

                    if (sbcService.getSelectedFilterTags() != "") {
                        filterTagTexts = sbcService.getSelectedFilterTags().split(",");
                    }

                    if (sbcService.getSelectedFilterText() != "") {
                        ftexts = sbcService.getSelectedFilterText().split(",");
                    } else if (sbcService.getSelectedMediaTypes() == "") {
                        // for the first time, getSelectedMediaTypes() will always be empty
                        ftexts = sbcService.getSelectedDefaultFilterText().split(",");
                        if(sbcService.getSelectedDefaultFilterTags() != "") {
                            //for now, no default tags are defined!
                            filterTagTexts = sbcService.getSelectedDefaultFilterTags().split(",");
                        }
                    }
                }

                $.each(docData.find("Point"), function(p, e) {

                    var pointName = $(e).attr("pointName"), pointType= $(e).attr("dataType"), isFilteredOut = false;
                    var pointTags = $(e).attr("tags");

                    //only show points selected in filter dialog
                    //and prevent tagged points from getting added in hierarchy
                    if (flags.split(",").indexOf(_pointNameMap[pointType]) == -1) {
                        isFilteredOut = true;
                    } else if (ftexts.length>0) {
                        $(ftexts).each(function() {
                            var t = (this.innerHTML) ? this.innerHTML : this;
                            if (t.toString().length<=pointName.length && pointName.toLowerCase().lastIndexOf(t.toLowerCase(),0) === 0) {
                                isFilteredOut = true;
                                return false; //break if point name starts with filter text (tag)
                            }
                        });
                    }

                    //we check for tags only if the point is not filtered out already.
                    // show points that contains at least 1 tag

                    if(!isFilteredOut && filterTagTexts.length > 0) {
                        var containsTag = false;
                        var pointTags = pointTags.split(",");
                        $(filterTagTexts).each(function () {
                            var t = (this.innerHTML) ? this.innerHTML : this;
                            $(pointTags).each(function(index, elm) {
                                if(elm == t.toString()) {
                                    containsTag = true;
                                    return false; //break if the point contains at least 1 tag.
                                }
                            });
                            if(containsTag) {
                                return false; //break if the point contains at least 1 tag.
                            }
                        });

                        // end of loop - point contains no tags specified by user, hide points that doesn't contain user specified tags
                        if(!containsTag) {
                            isFilteredOut = true;
                        }
                    }

                    if(isFilteredOut) {
                        return true; //continue and go to next datapoint (Point)
                    }

                    //add the point in hierarchy if not filtered out
                    var tokenize = pointName.split(".");
                    var dlevel = 0;
                    var ddata = _deviceMap[deviceName];
                    var dhrchy = _deviceHrchy[deviceName];
                    var i;

                    while(dlevel < tokenize.length-1) {
                        if(ddata[tokenize[dlevel]]) {
                            ddata = ddata[tokenize[dlevel]];
                            for(i = 0; i < dhrchy.children.length;i++) {
                                if(dhrchy.children[i].name === tokenize[dlevel]) {
                                    dhrchy = dhrchy.children[i];
                                    break;
                                }
                            }
                        } else {
                            ddata[tokenize[dlevel]] = {};
                            ddata = ddata[tokenize[dlevel]];
                            dhrchy.children.push({
                                name:tokenize[dlevel],
                                path:dhrchy.path + "." + tokenize[dlevel],
                                children:[]
                            });
                            dhrchy = dhrchy.children[dhrchy.children.length-1];
                        }
                        dlevel++;
                    }
                    if(!ddata.datapoints) {
                        ddata.datapoints = [];
                    }

                    var dataConversionClass = _getClass($(e).attr("dataType"));
                    ddata.datapoints.push({
                        select:deviceName+delimiter+pointName,
                        deviceName : deviceName,
                        pointName : pointName,
                        pointNameTitle : pointName,
                        pointNameClass:pointName,
                        memoryAddress:$(e).attr("memoryAddress"),
                        dataType: $(e).attr("dataType"),
                        comments:  $(e).attr("comments"),
                        commentsTitle:$(e).attr("comments"),
                        commentsClass:"comments",
                        dataFormat:"",
                        dataFormatClass:dataConversionClass,
                        tags:$(e).attr("tags"),
                        colorCode:$("#soe-datapoints-device-content .treeView >ul > li[data-key='"+deviceName+"']").index()%2
                    });
                });
                treeDeviceView.addChildren(currentTarget, _deviceHrchy[deviceName].children);
                treeDeviceView.expandTreeNode($(currentTarget).children("div").find(".expandcollapse"));
                //display hierarchy if we are retireving on device expansion
                _displayDatapointsInfo(_deviceMap[deviceName], currentTarget);
            };


            /**
             * _getAndDisplayDatapointsInfo - get datapoints for selected device by reading the corresponding device xml
             * and store & display read datapoints
             * @param currentTarget
             * @private
             */
            var _getAndDisplayDatapointsInfo = function (currentTarget) {

                var filename =  $(currentTarget).attr("data-deviceFile");


                //Note: check if there is device xml
                if(filename != null && filename.length > 0 && filename.trim() != "null") {
                    baja.Ord.make(filename).get({
                        ok:function(result) {
                            $.get(result.getReadUri(), function(data) {
                                var device = $(data).find("Device"), deviceName = $(device).attr("name");
                                _savedDpDocData[deviceName] = $(data);
                                _createAndDisplayDpInfo($(data), currentTarget, null);
                            });
                        }
                    });
                } else {
                    //simply remove the previous table  - if there is no datapoints to show, let the space be empty
                    $element.find("#soe-datapoints-display-content").empty();
                    selectedDeviceDatapoints = [];
                    $element.find("#soe-datapoints-displayed-label-count").text(selectedDeviceDatapoints.length);
                    $element.find(".treeView .spinner").removeClass("spinner");
                }
            };

            var _attachCheckboxEvents = function(){
                    if($("#soe-datapoints-device-content .treeView").find(".deviceOption:checked").length < $("#soe-datapoints-device-content .treeView").find(".deviceOption").length)
                        $(".soe-datapoints-device-header-selectAll.deviceOption").prop("checked",false);
                    else if($("#soe-datapoints-device-content .treeView").find(".deviceOption:checked").length == $("#soe-datapoints-device-content .treeView").find(".deviceOption").length)
                        $(".soe-datapoints-device-header-selectAll.deviceOption").prop("checked",true);
            };

            /**
             * _getDatapointsInfo: onclick handler of device (or any descendant in device hierarchy)
             * @param event
             * @private
             */
            var _getDatapointsInfo = function (event, isApply) {
                if($(event.target).hasClass("expandcollapse"))return;
                selectedDeviceDatapoints = [],selectedDevices = [];
                var selectedSubTree = null;
                if(event.currentTarget.getAttribute("data-devicefile") !== null)
                {
                    $(event.currentTarget).find(".deviceOption").prop("checked",!$(event.currentTarget).find(".deviceOption").prop("checked"));
                }
                else if(!$(event.currentTarget).hasClass("soe-datapoints-device-header-selectAll deviceOption"))
                {
                    $(event.currentTarget).closest("li[data-devicefile]").find(".deviceOption").prop("checked",false);
                    selectedSubTree = $(event.currentTarget)
                }
                _attachCheckboxEvents();
                selectedDevices = $("#soe-datapoints-device-content .treeView").find(".deviceOption:checked")
                $('#soe-datapoints-device-content .treeView .tree-node-selected').removeClass('tree-node-selected');
                if(selectedDevices.length > 0 || selectedSubTree !=null)
                {
                    var currentTarget,targetElement;
                    if(selectedSubTree != null)
                    {
                        var currentTarget = event.currentTarget; 
                        $(currentTarget).children("div").addClass('tree-node-selected spinner');
                        targetElement = event.target;
                        _getDataPointsEachDevice(currentTarget,targetElement,isApply);
                    }
                    $(selectedDevices).each(function()
                    {
                        var currentTarget = this.parentElement.parentElement;
                        this.parentElement.className = 'tree-node-selected spinner';
                        targetElement  = this;
                        _getDataPointsEachDevice(currentTarget,targetElement,isApply);
                    });
                }
                else 
                {
                    $element.find("#soe-datapoints-display-content .gcs_table_body").empty();
                    $element.find("#soe-datapoints-displayed-label-count").text("0"); 
                    selectedDeviceDatapoints = [];
                }
            };

            var _getDataPointsEachDevice = function(currentTarget,targetElement,isApply){
                $element.find(".sbcOfflineEngineering-actionButtons input").prop("disabled", true);

                var key = $(currentTarget).attr("data-key");
                if(_hasDatapointsData((key.split("."))[0])) {
                    //take the toElement key
                    var targetLI = $(targetElement).closest("li");
                    var targetKey = $(targetLI).attr('data-key');
                    var data = _getSelectedDeviceInfo(targetKey);
                    if (isApply && $(currentTarget).attr("data-deviceFile")) {
                        _createAndDisplayDpInfo(null, currentTarget, targetKey);
                    } else {
                        _displayDatapointsInfo(data, currentTarget);
                    }
                } else {
                    _getAndDisplayDatapointsInfo(currentTarget);
                }
                event.stopPropagation();
            }

            /**
             *
             * @private
             * get scanned list of devices to import from sbcOfflineEngineering service
             */
            var _getDeviceData = function() {
                var dataOrd = datapointsPath +"bql:select deviceName, deviceFile, ipAddress, stationNumber, numberOfDataPoints from sbcOfflineEngineering:SbcOfflineDevice order by deviceName";

                try {
                    baja.Ord.make(dataOrd)
                        .get({
                            ok: function (result) {
                            },
                            cursor: {
                                each: function () {
                                    var schematicsCursor = this;
                                    var slotPath = schematicsCursor.get("slotPath") ? schematicsCursor.get("slotPath") : "";
                                    var deviceName = schematicsCursor.get("deviceName") ? schematicsCursor.get("deviceName") : "";
                                    var deviceFile = schematicsCursor.get("deviceFile") ? schematicsCursor.get("deviceFile") : "";
                                    var ipAddress = schematicsCursor.get("ipAddress") ? schematicsCursor.get("ipAddress") : "";
                                    var stationNumber = schematicsCursor.get("stationNumber") ? schematicsCursor.get("stationNumber") : "";
                                    var totalDataPoints = schematicsCursor.get("numberOfDataPoints") ? schematicsCursor.get("numberOfDataPoints") : 0;
                                    var deviceNameTitle = lex.get("sbc.offlineEngineering.datapoints.ipAddress") +
                                        " " + ipAddress + "\n" +
                                        lex.get("sbc.offlineEngineering.datapoints.stationNumber") + " " + stationNumber;
                                    if(totalDataPoints!=0) {
                                        devicePointsData.data.push({
                                            "slotPath": slotPath,
                                            "deviceName": deviceName,
                                            "deviceNameTitle": deviceNameTitle,
                                            "deviceFile":deviceFile.$ord
                                        });
                                    }

                                },
                                after: function () {
                                    try {
                                        treeDeviceView = treeView.create({ data:devicePointsData.data,
                                            parent:$element.find("#soe-datapoints-device-content"),
                                        onclick: function(e){
                                            _getDatapointsInfo(e, false);
                                        }, options:{id:"soe-datapoints-device-content-treeView"}
                                        });

                                        //update total device count
                                        $element.find("#soe-datapoints-devices-label-count").text(
                                            $element.find("#soe-datapoints-device-content-treeView.treeView > ul > li").length);

                                        //display first device datapoints by default... highlight the selection!!
                                        $element.find("#soe-datapoints-device-content-treeView .tree-node:first").click();

                                        //enable select all devices
                                        //Note: commented code due to performance issue
                                        // $element.find("#soe-datapoints-device-selectall").on("click", _getAllDatapoints);

                                        //enable select all data points
                                        //Note: redundant functionality as the table itself supports select all datapoints option - remove later
/*
                                        $element.find("#soe-datapoints-datapoints-selectall").on("click", function(event) {
                                            //assume first column have checkbox selection
                                            var _selectColumn = this;
                                            var checkedState = $(_selectColumn).prop("checked");
                                            var allDatapoints = $("#soe-datapoints-display-table")
                                                .find("td:nth-child(1) input[type='checkbox']");

                                            console.log("select All status:  " + checkedState);
                                            $.each($(allDatapoints), function (index, element) {
                                                $(element).prop("checked", checkedState);
                                                _addDeleteSelectedDatapointsList(event, element);
                                            });
                                        });
*/
                                    } catch (e) {
                                        //    logWarning(e);
                                    }
                                },
                                limit: 1000, //scannedList.datapoints /* get scanned datapoints count!! */
                                offset: 0
                            }
                        });

                } catch (e) {
                    console.log(e);
                }

            };

            var _resetData = function () {
                scannedList = {};
                selectedModules = {};
                devicePointsData = {data:[]};
                _deviceMap = {};
                _deviceHrchy = {};
                overwriteDialog = null;
                selectedDatapointsList = {};
                selectedDeviceDatapoints = [];
                selectTableCurrentPointer = 0;
                displayTableCurrentPointer = 0;
                if (_filterDpDialog) {
                    _filterDpDialog.close();
                }
                _filterDpDialog = null;

            };

            /**
             * _resetAndUpdateConfigSelection: prompt "preparing for import..." dialog box and update selected datapoints to
             * sbcOfflineEngineering service
             * @returns {*}
             * @private
             */
            var  _resetAndUpdateConfigSelection = function() {

                //reset datapoints selection before update
                var datapointsContainer  = sbcService.getDataPoints();
                datapointsContainer.setSelectedDataPointsCount(0);
                var selectedList = {};
                if(scannedList.datapoints) {
                    selectedList = selectedDatapointsList;
                }
                return (selectedList);
            };

            /**
             * _updateConfigSelectionInChunks: create baja component for user selected datapoints
             * @param selectedList
             * @returns {Array}
             * @private
             */

            var _updateConfigSelectionInChunks = function (selectedList, lowerLimit, upperLimit, resolveNext) {
                var mainComponent, datapointComponent, tempComponent;
                var i, selectedListKeys = Object.keys(selectedList);

                mainComponent = new baja.Component();
                datapointComponent = new baja.Component();
                var startIndex = 0;
                tempComponent = new baja.Component();
                //monitorConfigurationsTimer(resolveNext);

                for (i = lowerLimit; i < upperLimit; i++) {
                    startIndex = startIndex+1;
                    tempComponent.add({slot:'device'+startIndex, value:selectedList[selectedListKeys[i]].device});
                    tempComponent.add({slot:'pointName'+startIndex, value:selectedList[selectedListKeys[i]].pointName});
                    tempComponent.add({slot:'selected'+startIndex, value:selectedList[selectedListKeys[i]].selected});
                    tempComponent.add({slot:'overwrite'+startIndex, value:selectedList[selectedListKeys[i]].overwrite});
                    tempComponent.add({slot:'dataConversion'+startIndex, value:selectedList[selectedListKeys[i]].dataFormat});
                    tempComponent.add({slot:'mediaType'+startIndex, value:selectedList[selectedListKeys[i]].mediaType});
                    tempComponent.add({slot:'memoryAddress'+startIndex, value:selectedList[selectedListKeys[i]].memoryAddress});
                    tempComponent.add({slot:'comments'+startIndex, value:((selectedList[selectedListKeys[i]].comments) ? selectedList[selectedListKeys[i]].comments : "")});
                    //tempComponent.add({slot:'comments', value:"swamy"});

                    //datapointComponent.add(tempComponent);

                }
                //console.log(lowerLimit+"...."+upperLimit+"..."+i);
                mainComponent.add( {slot:"datapoints", value:tempComponent});
                mainComponent.add({slot:"totalRecords",value:startIndex})
                var startPosition = "1",endPosition="0";
                if(lowerLimit==0)
                    startPosition = "0"

                if(upperLimit==selectedListKeys.length)
                    endPosition = "1"

                mainComponent.add( {slot:"startPosition", value:startPosition});
                mainComponent.add( {slot:"endPosition", value:endPosition});
                sbcService.invoke({
                    slot:"updateDataPointsInformation",
                    value:mainComponent
                }).then(function(){
                    if(upperLimit < selectedListKeys.length) {

                        lowerLimit = upperLimit;
                        upperLimit = (selectedListKeys.length < (upperLimit+_configUpdateLimit)) ? selectedListKeys.length : (upperLimit+_configUpdateLimit);
                        window.setTimeout(_updateConfigSelectionInChunks, 500, selectedList, lowerLimit, upperLimit, resolveNext)
                    } else {
                        console.log("Done with update service, waiting to set dataPointsConfigureState to false.");
                        _resolveNext = resolveNext;
                    }
                });
            };

            /**
             * _updateConfigData: reset previously selected datapoints config if any and update currently selected datapoints
             * @param updateStatus
             * @returns {*}
             * @private
             */
            var _updateConfigData = function (resolveNext) {
                var selectedList = _resetAndUpdateConfigSelection();
                var selectedListKeys = Object.keys(selectedList);
                var lowerLimit = 0, upperLimit = (selectedListKeys.length < _configUpdateLimit) ? selectedListKeys.length : _configUpdateLimit;
                _updateConfigSelectionInChunks(selectedList, lowerLimit, upperLimit, resolveNext);
            };

            /**
             *
             * @param resolve
             * @param hasMoreOverwritePrompts
             * @param keys
             * @param index
             * @private
             */
            var _promptOverwriteDialog = function (resolve, hasMoreOverwritePrompts, keys, index) {

                var content = "<div class='soe-overwrite-dialog'><div>"+
                    lex.get("sbc.offlineEngineering.datapoints.overwrite")+"</div>" +
                    "<div>"+lex.get("sbc.offlineEngineering.datapoints.overwrite.yes")+"</div>" +
                    "<div>"+lex.get("sbc.offlineEngineering.datapoints.overwrite.no")+"</div>" +
                    "<div> <input type='checkbox' id='soe-datapoints-overwrite-check' >" +
                    "<label for='soe-datapoints-overwrite-check'>" +
                    lex.get("sbc.offlineEngineering.datapoints.overwrite.repeat")+"</label>" +
                    "</div></div>";

                content = content.replace("$$deviceName$$", (selectedDatapointsList[keys[index]]).device)
                    .replace("$$pointName$$", (selectedDatapointsList[keys[index]]).pointName);

                overwriteDialog =  dialogs.make(
                    {
                        title : lex.get("sbc.offlineEngineering.dialog.confirm"),
                        content:content,
                        buttons: [
                            {
                                name:"overwrite",
                                displayName: lex.get("sbc.offlineEngineering.dialog.overwrite"),
                                handler: function(dlg) {
                                    var dlgContent = dlg.content();
                                    dlg.close();
                                    // second import, so "isFirstImport" is false
                                    importDataPoints(dlgContent, keys, index, hasMoreOverwritePrompts, resolve, false);
                                }
                            },
                            {
                                name:"skip",
                                displayName:lex.get("sbc.offlineEngineering.dialog.skip"),
                                handler:function (dlg) {
                                    selectedDatapointsList[keys[index]].overwrite='0';
                                    index++;
                                    dlg.close();
                                    var dlgContent = dlg.content();
                                    var repeatNo = $(dlgContent).find("#soe-datapoints-overwrite-check").prop("checked");
                                    if(repeatNo || index >= keys.length ) {

                                        _prepImportDialog = dialogs.show({
                                            title:lex.get("sbc.offlineEngineering.dialog.info"),
                                            content: lex.get("sbc.offlineEngineering.dialog.prepareforimport")});

                                        window.setTimeout(function () {
                                            while (index < keys.length) {
                                                selectedDatapointsList[keys[index++]].overwrite='0';
                                            }

                                            var resolveNext = function() {
                                                if(_prepImportDialog && !_prepImportDialog.isClosed()) {
                                                    _prepImportDialog.close();
                                                }
                                                resolve();
                                            };

                                            var onUpdatePromise = _updateConfigData(resolveNext);
                                        }, 0, index, keys, resolve);
                                    } else {
                                        _promptOverwriteDialog(resolve, hasMoreOverwritePrompts, keys, index);
                                    }
                                }
                            }
                        ]
                    });
                sbcService.invoke({
                    slot: "scanExistDevices"
                }).then(function (returnValue) {
                    if (returnValue) {
                        overwriteDialog.show({content: content});
                    } else {
                        // first import, so "isFirstImport" is true
                        importDataPoints(content, keys, index, hasMoreOverwritePrompts, resolve, true);
                    }
                });
            };

            /**
             *
             * @param dlgContent
             * @param keys
             * @param index
             * @param hasMoreOverwritePrompts
             * @param resolve
             * @param isFirstImport
             * @private
             */
            function importDataPoints(dlgContent, keys, index, hasMoreOverwritePrompts, resolve, isFirstImport) {
                selectedDatapointsList[keys[index]].overwrite = '1';
                index++;
                var repeatYes = $(dlgContent).find("#soe-datapoints-overwrite-check").prop("checked");
                if (repeatYes || index >= keys.length || isFirstImport) {
                    _prepImportDialog = dialogs.show({
                        title: lex.get("sbc.offlineEngineering.dialog.info"),
                        content: lex.get("sbc.offlineEngineering.dialog.prepareforimport")
                    });
                    window.setTimeout(function (index, keys, resolve) {

                        while (index < keys.length) {
                            selectedDatapointsList[keys[index++]].overwrite = '1';
                        }
                        var resolveNext = function () {
                            if (_prepImportDialog && !_prepImportDialog.isClosed()) {
                                _prepImportDialog.close();
                            }
                            resolve();
                        };

                        var onUpdatePromise = _updateConfigData(resolveNext);
                    }, 0, index, keys, resolve);

                } else {
                    _promptOverwriteDialog(resolve, hasMoreOverwritePrompts, keys, index);
                }
            }

            /**
             *This method sets datapoints filter criteria in dialog box
             * @param resolve
             * @private
             */
            var _setDpFilterCriteria = function (mediaTypes, filterText, filterTags) {

                $("#dpFilter-selectall").prop("checked",false);
                $(".pointInFilter").prop("checked",false);
                $('.dpTag').remove();
                $('#hideTags>img').remove();

                //cleanup tags before reattaching them
                $('.dpTrueTags').remove();
                $('#showTags>img').remove();


                if (mediaTypes == "") {
                    mediaTypes = sbcService.getSelectedDefaultMediaTypes();
                }

                if (sbcService.getIsDefaultFilter() && filterText == "") {
                    filterText = sbcService.getSelectedDefaultFilterText();
                }

                //tick media types
                var typesArr = mediaTypes.split(",");
                typesArr.forEach(function(v) {
                    if(!isNaN(parseInt(v))) {
                        $('#dpFilter-' + v).prop("checked", true);
                    }
                });
                if(typesArr.length == 7) {
                    $("#dpFilter-selectall").prop("checked",true);
                }

                //add filter tag texts
                if (filterText != "") {
                    filterText.split(",").forEach(function(t) {
                        var $span = $('<span class="dpTag">');
                        $span[0].innerText= t;
                        $('#hideTags>input').before($span);
                    });
                }

                // TODO: Are we going to save user specified tags in station?
                // filterTags

                if (filterTags && filterTags != "") {
                    filterTags.split(",").forEach(function(t) {
                        var $span = $('<span class="dpTrueTags">');
                        $span[0].innerText= t;
                        $('#showTags>input').before($span);
                    });
                }

                //add grey plus icon
                $('#hideTags>input').before('<img src="/module/sbcOfflineEngineering/rc/images/+_gray.svg"/>');
                $('#hideTags>input, #hideTags>img').on("click", function () {
                    $('#hideTags>img').remove();
                    $('#hideTags>input').focus();
                });

                // Add grey plus icon
                $('#showTags>input').before('<img src="/module/sbcOfflineEngineering/rc/images/+_gray.svg"/>');
                $('#showTags>input, #showTags>img').on("click", function () {
                    $('#showTags>img').remove();
                    $('#showTags>input').focus();
                });


            };

            /**
             *This method validates filter criteria before applying it
             * @private
             */

            var _validateAndSaveFilter = function() {
                var inputTexts = "", inputTypes = "", isError = false;
                var inputTagTexts = "";
                if(!$(".pointInFilter").is(':checked')) {
                    dialogs.showOk({
                        title : lex.get("sbc.offlineEngineering.dialog.error"),
                        content:lex.get("sbc.offlineEngineering.dpFilter.selecterror")
                    });
                    isError = true;
                } else {
                    $(".dpTag").each(function() {
                        if (this.innerHTML.indexOf(',') !== -1) {
                            dialogs.showOk({
                                title : lex.get("sbc.offlineEngineering.dialog.error"),
                                content:lex.get("sbc.offlineEngineering.dpFilter.invalid")
                            });
                            isError = true;
                        } else {
                            inputTexts+= this.innerText+",";
                        }
                    });

                    $(".dpTrueTags").each(function() {
                        if (this.innerHTML.indexOf(',') !== -1) {
                            dialogs.showOk({
                                title : lex.get("sbc.offlineEngineering.dialog.error"),
                                content:lex.get("sbc.offlineEngineering.dpFilter.invalidTags")
                            });
                            isError = true;
                        } else {
                            inputTagTexts += this.innerText+",";
                        }
                    });

                }

                if (isError){
                    return false;
                } else {
                    if(inputTexts.length > 0) {
                        inputTexts = inputTexts.substring(0,inputTexts.length-1);
                    }
                    sbcService.setSelectedFilterText(inputTexts);

                    if(inputTagTexts.length > 0) {
                        inputTagTexts = inputTagTexts.substring(0, inputTagTexts.length-1);
                    }
                    sbcService.setSelectedFilterTags(inputTagTexts);

                    $(".pointInFilter").each(function() {
                        if(this.checked) {
                            inputTypes+= this.id.charAt(this.id.length-1) + ",";
                        }
                    });

                    inputTypes = inputTypes.substring(0,inputTypes.length-1);
                    sbcService.setSelectedMediaTypes(inputTypes);

                    if (inputTypes == sbcService.getSelectedDefaultMediaTypes() && inputTexts == sbcService.getSelectedDefaultFilterText()
                            && inputTagTexts == sbcService.getSelectedDefaultFilterTags()) {
                        sbcService.setIsDefaultFilter(true);
                        $("#soe-datapoints-filterButton").attr("src","/module/sbcOfflineEngineering/rc/images/filter_icon.svg");
                    } else {
                        sbcService.setIsDefaultFilter(false);
                        $("#soe-datapoints-filterButton").attr("src","/module/sbcOfflineEngineering/rc/images/filter_icon_blue.svg");
                    }

                    return true;
                }

            };

            var _applyFilter = function() {
                try {
                    var currLI = $element.find(".tree-node-selected").closest("li[data-deviceFile]"), currKey = $(currLI).attr("data-key"), scrollPos = $element.find(".treeView").prop("scrollTop");
                    $element.find(".treeView").remove();
                    treeDeviceView = treeView.create({ data:devicePointsData.data,
                        parent:$element.find("#soe-datapoints-device-content"),
                        onclick: function(e) {
                            _getDatapointsInfo(e,true);
                        }, options:{id:"soe-datapoints-device-content-treeView"}
                    });

                    //update total device count
                    $element.find("#soe-datapoints-devices-label-count").text(
                    $element.find("#soe-datapoints-device-content-treeView.treeView > ul > li").length);

                    //display previously selected device after filter or else first by default & highlight the selection
                    if (currKey) {
                        $element.find("li[data-key='" + currKey + "'] > div .expandcollapse").click();
                        $element.find(".treeView").prop("scrollTop", scrollPos);
                    } else {
                        $element.find("#soe-datapoints-device-content-treeView .tree-node:first > div .expandcollapse").click();
                    }


                } catch (e) {
                    console.log(e);
                }
            };

            /**
             *This method create and displays dialog to filter datapoints
             * @private
             */
            var _promptFilterDialog = function () {
                if (_filterDpDialog==null) {
                    _filterDpDialog = dialogs.show({
                        title: "",
                        content: filterTemplate({
                            filterByLabel: lex.get("sbc.offlineEngineering.dpFilter.filterBy"),
                            dpFilterSelectAll: lex.get("sbc.offlineEngineering.dpFilter.dpFilterSelect"),
                            registersLabel: lex.get("sbc.offlineEngineering.dpFilter.registers"),
                            flagsLabel: lex.get("sbc.offlineEngineering.dpFilter.flags"),
                            countersLabel: lex.get("sbc.offlineEngineering.dpFilter.counters"),
                            iosLabel: lex.get("sbc.offlineEngineering.dpFilter.ios"),
                            timersLabel: lex.get("sbc.offlineEngineering.dpFilter.timers"),
                            dbLabel: lex.get("sbc.offlineEngineering.dpFilter.db"),
                            textLabel: lex.get("sbc.offlineEngineering.dpFilter.text"),
                            hideLabel: lex.get("sbc.offlineEngineering.dpFilter.hide"),
                            showTagsLabel: lex.get("sbc.offlineEngineering.dpFilter.showTags")
                        }),
                        buttons:[
                            {
                                name:"restore",
                                displayName:lex.get("sbc.offlineEngineering.dpFilter.restore"),
                                handler:function(dialog) {
                                    _setDpFilterCriteria(sbcService.getSelectedDefaultMediaTypes(), sbcService.getSelectedDefaultFilterText(), sbcService.getSelectedDefaultFilterTags());
                                    return false;
                                }
                            },
                            {
                                name:"apply",
                                displayName:lex.get("sbc.offlineEngineering.dpFilter.apply"),
                                handler: function(dialog) {
                                    if(_validateAndSaveFilter()) {
                                        _applyFilter();
                                        dialog.hide();
                                    }
                                    return false;
                                }
                            }
                        ]
                    });

                    $('.js-dialog').has('#dpFilter-box').css("background", '#eee');
                    $('ul.js-dialog-button-content').has('.js-dialog-button-restore').css("background", '#eee');

                    $('#closeFilter').on('click', function() {
                        _filterDpDialog.hide();
                    });

                    $("#dpFilter-selectall").on('click',function() {
                        $(".pointInFilter").each(function() {
                            $(this).prop("checked",$("#dpFilter-selectall").is(':checked'));
                        });
                    });

                    $(".pointInFilter").on('click', function () {
                        var checkedAll = true;
                        $(".pointInFilter").each(function (index, e) {
                            if(!this.checked) {
                                checkedAll = false;
                                return false;
                            }
                        });
                        $("#dpFilter-selectall").prop("checked", checkedAll);
                    });

                    $('#hideTags input').bind('keyup focusout',function(e) {
                        if (e.type == "keyup") {
                            var keyCode = e.keyCode || e.which;
                        }
                        if (e.type == "focusout" || keyCode == 13) {
                            var txt = $.trim( this.value );
                            if(txt) {
                                var $span = $('<span class="dpTag">');
                                $span[0].innerText = txt;
                                $(this).before($span);
                                $(this).focus();
                            }
                            this.value = "";
                        }
                    });

                    $('#hideTags').on('click','.dpTag', function() {
                        $(this).remove();
                    });

                    // logic to add more tags in filter dialog - tags refer to 'tags' field not 'datapoints namespace'!
                    $('#showTags input').bind('keyup focusout',function(e) {
                        if (e.type == "keyup") {
                            var keyCode = e.keyCode || e.which;
                        }
                        if (e.type == "focusout" || keyCode == 13) {
                            var txt = $.trim( this.value );
                            if(txt) {
                                var $span = $('<span class="dpTrueTags">');
                                $span[0].innerText = txt;
                                $(this).before($span);
                                $(this).focus();
                            }
                            this.value = "";
                        }
                    });

                    $('#showTags').on('click','.dpTrueTags', function() {
                        $(this).remove();
                    });


                } else {
                    _filterDpDialog.show();
                }

                _setDpFilterCriteria(sbcService.getSelectedMediaTypes(), sbcService.getSelectedFilterText(), sbcService.getSelectedFilterTags());

            };

            var _resizeSelectedTable = function () {
                var selectedTable = $("#soe-datapoints-selected-table");

                var theadWidth = selectedTable.find("thead").width();
                var trWdith = selectedTable.find("tr:first").width();
                var computedWidth = 0;
                $.each(selectedTable.find("tr:first").children(), function (index, element) {
                    computedWidth += $(element).width();
                });

                if(theadWidth != computedWidth) {
                    selectedTable.find("tr > th:nth-child(1)").css("width", (theadWidth * .67));
                    selectedTable.find("tr > th:nth-child(1)").css("min-width", (theadWidth * .67));
                    selectedTable.find("tr > td:nth-child(1)").css("width", (theadWidth * .67));

                    selectedTable.find("tr > th:nth-child(2)").css("width", (theadWidth * .23));
                    selectedTable.find("tr > th:nth-child(2)").css("min-width", (theadWidth * .23));
                    selectedTable.find("tr > td:nth-child(2)").css("width", (theadWidth * .23));

                    selectedTable.find("tr > th:nth-child(3)").css("width", (theadWidth * .12));
                    selectedTable.find("tr > th:nth-child(3)").css("min-width", (theadWidth * .12));
                    selectedTable.find("tr > td:nth-child(3)").css("width", (theadWidth * .12));
                }
            };


            var _attachSubscriber = function(prop, cx) {
                var service = this;
                var propName = prop.getName();
                if (propName === "dataPointsConfigureState" && !service.getDataPointsConfigureState()){
                    console.log("dataPointsConfigureState set to false, moving to next View!!");
                    //unsubscribe
                    subscriber.detach("changed", _attachSubscriber);
                    if(_resolveNext) {
                        _resolveNext();
                    }
                }
            };


            return {


                /**
                 *
                 * @param element: dom element the view will be added to
                 * @param params: javascript object with widget parameters required to load the view within the widget
                 */

                loadView : function(element, params) {
                    _resetData();
                    $element = $(element);
                    $widgetContainer = (params.widgetContainer) ?  $(params.widgetContainer) : (element).closest(".sbcOfflineEngineering-wizard");
                    selectedModules = params.selectedModules;
                    scannedList = params.scannedModules;
                    sbcService = params.component;

                    //append default html layout template for datapoints selection screens
                    $(element).append(template({
                        defaultActionMsg: lex.get("sbc.offlineEngineering.datapoints.defaultMsg"),
                        deviceSelectAll:lex.get("sbc.offlineEngineering.datapoints.deviceSelectAll"),
                        datapointsDeviceCount:lex.get("sbc.offlineEngineering.datapoints.datapointsDeviceCount"),
                        datapointsSelectAll:lex.get("sbc.offlineEngineering.datapoints.datapointsSelectAll"),
                        datapointsDatapointsCount:lex.get("sbc.offlineEngineering.datapoints.datapointsDatapointsCount"),
                        datapointsSelectedPoints:lex.get("sbc.offlineEngineering.datapoints.datapointsSelectedPoints"),
                        datapointsDeleteAll:lex.get("sbc.offlineEngineering.datapoints.datapointsDeleteAll"),
                        pointName:lex.get("sbc.offlineEngineering.datapoints.pointName"),
                        expandCollapseMsg:lex.get("sbc.offlineEngineering.datapoints.svg.altText"),
                        searchPoint: lex.get("sbc.offlineEngineering.datapoints.searchPointLabel"),
                        device: lex.get("sbc.offlineEngineering.datapoints.device")
                    }));

                    //handler to maximize and restore datapoints display table for readability
                    $element.find("#soe-datapoints-display-restore-max").on('click', function() {
                        var restoreMaxIcon = this;
                        if($(restoreMaxIcon).hasClass("restored")) {
                            $element.find("#soe-datapoints-selection-datapoints").switchClass("displayDiv animateWidth31", "hideDiv", 10);
                            $element.find("#soe-datapoints-display-datapoints").switchClass("hideDiv animateWidth44", "displayDiv animateWidth75", 10);
                            $(restoreMaxIcon).removeClass("restored");
                            $(restoreMaxIcon).addClass("maximized");
                            $(restoreMaxIcon).attr('src', '/module/sbcOfflineEngineering/rc/images/restore.svg');
                        } else if($(restoreMaxIcon).hasClass("maximized")) {
                            $element.find("#soe-datapoints-display-datapoints").switchClass("animateWidth75", "animateWidth44", 10);
                            $element.find("#soe-datapoints-selection-datapoints").switchClass("hideDiv","displayDiv animateWidth31", 10);
                            $element.find("#soe-datapoints-selection-datapoints").removeAttr("style");
                            $(restoreMaxIcon).removeClass("maximized");
                            $(restoreMaxIcon).addClass("restored");
                            $(restoreMaxIcon).attr('src', '/module/sbcOfflineEngineering/rc/images/maximize.svg');
                        }
                    });

                    //handler to maximize and restore datapoints selection table for readability
                    $element.find("#soe-datapoints-selection-restore-max").on('click', function() {
                        var restoreMaxIcon = this;
                        if($(restoreMaxIcon).hasClass("restored")) {
                            $element.find("#soe-datapoints-display-datapoints").switchClass("displayDiv animateWidth44", "hideDiv", 10);
                            $element.find("#soe-datapoints-selection-datapoints").switchClass("hideDiv animateWidth31", "displayDiv animateWidth75", 10);
                            $(restoreMaxIcon).removeClass("restored");
                            $(restoreMaxIcon).addClass("maximized");
                            $(restoreMaxIcon).attr('src', '/module/sbcOfflineEngineering/rc/images/restore.svg');


/*
                            window.setTimeout(function () {
                                _resizeSelectedTable();
                            }, 100);
*/

                        } else if($(restoreMaxIcon).hasClass("maximized")) {
                            $element.find("#soe-datapoints-selection-datapoints").switchClass("animateWidth75", "animateWidth31", 10);
                            $element.find("#soe-datapoints-display-datapoints").switchClass("hideDiv", "displayDiv animateWidth44", 10);
                            $(restoreMaxIcon).removeClass("maximized");
                            $(restoreMaxIcon).addClass("restored");
                            $(restoreMaxIcon).attr('src', '/module/sbcOfflineEngineering/rc/images/maximize.svg');

/*
                            window.setTimeout(function () {
                                var selectedTable = $("#soe-datapoints-selected-table");
                                selectedTable.find("tr > th").css("width", "");
                                selectedTable.find("tr > th").css("min-width", "");
                                selectedTable.find("tr > td").css("width", "");
                            }, 100);
*/
                        }
                    });

                    //create dialog box for datapoints filtering in left navigation
                    $element.find("#soe-datapoints-filterButton").on('click', _promptFilterDialog);

                    selectTableCurrentPointer = 0;
                    selectTableScroll = nscroll({ getContent:_getSelectTableTRContent,
                            onLoad: null,
                            onUnload: null,
                            updateLimit: 50,
                            loadLimit:tableLimits.updateLimit,
                            limitedVisibility:true,
                            visibleLimit:tableLimits.visibleLimit,
                            parent:$element.find("#soe-datapoints-selected-list"),
                            contentSelector:'#soe-datapoints-selected-table  tbody',
                            content:"<table class='ux-table no-stripe' id='soe-datapoints-selected-table'>"
                            +"<thead class='ux-table-head'><tr><th>"+lex.get("sbc.offlineEngineering.datapoints.pointName")
                            +"</th><th>"+lex.get("sbc.offlineEngineering.datapoints.device")+"</th>"
                            +"<th></th></tr></thead><tbody class='gcs_table_body' tabindex=-1></tbody></table>",
                            contentSize:Object.keys(selectedDatapointsList).length
                        });

                    //handler to delete item from selected datapoints list
                    //Note: Attach handler only once - as it delegates the deletion to table we don't have to attach it multiple times
                    $element.find("#soe-datapoints-selected-table").on("click", function (event) {
                        //check for delete only
                        var tokenizer, selectedDPCount, selectedListDisplayLength;
                        var target = event.target;

                        if($(target).hasClass("soe-delete-selected-icon")) {
                            var parentId = $(target).parent().attr("id");
                            tokenizer = parentId.split(delimiter);
                            $("#soe-datapoints-selected-table").find("tr[id='"+utils.escapeJQID(parentId)+"']").remove();
                            var selectedInput = $("#soe-datapoints-display-table")
                                .find("td.gcs_td_cell input[type='checkbox'][value='"+parentId+"']");
                            $(selectedInput).prop("checked", false);
                            delete selectedDatapointsList[parentId];

                            //decrement selectTable counter that tracks datapoints displayed in select table
                            selectTableCurrentPointer--;

                            //uncheck the display table header
                            var displayedTR = $("#soe-datapoints-display-table tbody tr");
                            if(displayedTR.length > displayedTR.find("input[type='checkbox']:checked").length) {
                                $element.find("#soe-datapoints-display-table th input[type='checkbox']").prop("checked", false);
                            }

                            selectedDPCount = Object.keys(selectedDatapointsList).length;
                            //update selectedDatapointscount for scroll
                            selectTableScroll.updateOptions({
                                contentSize:selectedDPCount
                            });

                            selectedListDisplayLength = $element.find("#soe-datapoints-selected-table > tbody").children().length;

                            if((selectedListDisplayLength < selectedDPCount) && selectedListDisplayLength < tableLimits.visibleLimit) {
                                selectTableScroll.loadContent();
                            }

                            //update selected points count
                            $element.find("#soe-datapoints-selected-label-count").text(selectedDPCount);
                            event.stopPropagation();
                        }

                    });


                    $("#soe-datapoints-selected-table  tbody").scroll(function (e) {
                        var _self = this;
                        var left = $(_self).scrollLeft();
                        $("#soe-datapoints-selected-table .ux-table-head").scrollLeft($(_self).scrollLeft());
                    });


                    _getDeviceData();

                    //for searching datapoints
                    $("#soe-datapoints-searchBox > input").on("keyup", function() {
                        if(typingTimeout != null) clearTimeout(typingTimeout);
                        typingTimeout = setTimeout(_searchByDpName,150);
                    });

                    //change filter icon color according to filter criteria
                    if(sbcService.getIsDefaultFilter()){
                        $("#soe-datapoints-filterButton").attr("src","/module/sbcOfflineEngineering/rc/images/filter_icon.svg");
                    } else {
                        $("#soe-datapoints-filterButton").attr("src","/module/sbcOfflineEngineering/rc/images/filter_icon_blue.svg");
                    }
                },

                /**
                 * validate user input
                 *
                 * @returns {boolean}: return true/false based on validity of user input
                 */
                 validate: function() {
                 //validate user input
                    return new Promise(function (resolve) {
                        if($element.find("#soe-datapoints-selected-table tbody > tr").length <= 0) {
                            dialogs.showOk({
                                title : lex.get("sbc.offlineEngineering.dialog.error"),
                                content:lex.get("sbc.offlineEngineering.datapoints.selectinput")
                            });
                            resolve(false);
                        } else {
                            resolve(true);
                        }
                    });
                 },

                /**
                 * clean up and save before going to next view
                 *
                 * @returns {Promise}: return promise to be resolved before going to next view
                 */
                next : function(event, params) {
                    subscriber = params.subscriber;
                    subscriber.attach('changed', _attachSubscriber);
                    var datapointsContainer  = sbcService.getDataPoints();
                    subscriber.subscribe(datapointsContainer);

                    return new Promise(function(resolve){
                        //TODO:task to do before moving to next screen

                        var hasMoreOverwritePrompts = true;
                        var index = 0;
                        var keys = Object.keys(selectedDatapointsList);

                        //reference to resolve in subscriber 'changed' method when dataPointsConfigureState is false;
                        _resolveNext = resolve;
                        //TODO: remove resolve as function argument as its global and no need to passed around.
                        _promptOverwriteDialog(resolve, hasMoreOverwritePrompts, keys, index);

                        //resolve();

                    });
                },

                /**
                 * clean up and save before going to previous view
                 *
                 */
                back : function() {
                    return new Promise(function(resolve){
                        //TODO:task to do before moving to previous screen
                        resolve();
                    });
                },

                /**
                 * cancel operation
                 *
                 */
                 cancel: function() {
                 //Note: for DatapointsView cancel is similar to reset selection functionality
                    //remove all selected datapoints <tr>
                    $element.find("#soe-datapoints-selected-table > tbody > tr").remove();
                    //uncheck points for the selected device in display table
                    $element.find("#soe-datapoints-display-table th > input[type='checkbox']").prop("checked", false);
                    $element.find("#soe-datapoints-display-table  > tbody > tr td.gcs_td_cell input[type='checkbox']")
                        .prop("checked", false);
                    //clear selected datapoints list saved internally
                    selectedDatapointsList = {};

                    //reset selected points count to zero
                    $element.find("#soe-datapoints-selected-label-count").text(0);
                    selectTableScroll.updateOptions({contentSize:0});
                    selectTableCurrentPointer = 0;

                },

                /**
                 * data to be passed to widget to communicate between views
                 *
                 * @returns {json}: return key value pairs of data
                 */
                getData: function() {
                    //Note: task to update data to be passed to widget
                    return {};
                },


                /**
                 * enable/disable status of visisble action buttons within the view
                 *
                 * @returns {json}: return action buttons status (enabled/disabled)
                 */
                /*
                 //Note: by default all buttons within the view are enabled.
                 // uncomment and update the code to conditionally enable or disable action buttons
                 enableDisableActionBttns: function () {
                 //Note: one can update action buttons status
                 return {
                 disabledBack : false,
                 disabledNext : false,
                 disabledCancel:false
                 };
                 },
                 */
                params:{
                    vname:'datapointsView',
                    id:'datapointsView',
                    message: lex.get('sbc.offlineEngineering.datapoints.defaultMsg'),
                    backBttnMsg: lex.get('sbc.offlineEngineering.back'),
                    nextBttnMsg: lex.get('sbc.offlineEngineering.next'),
                    isCancelAvailable: true,
                    cancelBttnMsg: lex.get('sbc.offlineEngineering.datapoints.datapointsResetSelection')
                }
            };
        }();

        return datapointsView;
    });