/**
 * * Copyright (c) 2018. Honeywell International, Inc. All Rights Reserved.
 */

define(['baja!',
        'jquery',
        'Promise',
        'underscore',
        'dialogs',
        'nmodule/sbcOfflineEngineering/rc/js/util/Progress',
        'nmodule/galileoWidgets/rc/js/GalileoUITable',
        'nmodule/sbcOfflineEngineering/rc/js/util/utils',
        'nmodule/sbcOfflineEngineering/rc/js/nscroll',
        'hbs!nmodule/sbcOfflineEngineering/rc/hbs/finishView',
        'hbs!nmodule/sbcOfflineEngineering/rc/hbs/detailedReport',
        'hbs!nmodule/sbcOfflineEngineering/rc/hbs/reportTable',
        'hbs!nmodule/sbcOfflineEngineering/rc/hbs/reportTable-Row',
        'lex!sbcOfflineEngineering'
    ],
    function (baja,
              $,
              Promise,
              _,
              dialogs,
              progress,
              uiTable,
              utils,
              nscroll,
              template,
              reportTemplate,
              reportTable,
              reportTableRow,
              Lexicons) {

        'use strict';

        /**
         * Description of your widget.
         *
         * @class
         * @alias module:nmodule/sbcOfflineEngineering/rc/js/OfflineEngineeringfinishView
         */
        var finishView = new function () {

                var lex = Lexicons[0];
                var $element = null;
                var selectedModules = {};
                var importStatus = {};
                var importList = {datapoints: 0, alarms: 0, hdlogs: 0};
                var sbcService = null;
                var $widgetContainer = null;
                var devicePath = "file:^offline/engineering/system.bog|bog:|slot:|";
                var deviceData = {data: []};
                var reportPopupContent = "";
                var reportPopupDialog = null;

                //derived from css class .ux-table > tbody td
                var _TD_HGHT = 31;

                var _resetData = function () {
                    selectedModules = {};
                    importStatus = {};
                    importList = {datapoints: 0, alarms: 0, hdlogs: 0};
                };

                var tableLimits = {
                    visibleLimit:100,
                    updateLimit:50
                };

                var _spin = "<div class='spinBlue'></div>";
                var expandAllDevices = -1;

                var datapointsReportDetails = {};

                var _getDatapointsTRContent = function (deviceName, start, end) {
                    var deviceDetails = datapointsReportDetails[deviceName];
                    var l1 = (start  != undefined) ? start : deviceDetails.currentpointer,
                        l2 = (end != undefined) ? end : (deviceDetails.currentpointer+tableLimits.updateLimit);
                    deviceDetails.currentpointer = l2;

                    var e = reportTableRow({
                        columns: deviceDetails.columns,
                        rows: deviceDetails.rows.slice(l1, l2)
                    });
                    return e.trim();
                };

                var _createDatapointsReport = function (deviceName, columns, rows, parent) {

                    var displayTableScroll = nscroll({ getContent:function(deviceName){
                            return function(start, end){return _getDatapointsTRContent(deviceName, start, end);}}(deviceName),
                        onLoad: null,
                        onUnload: null,
                        updateLimit: 50,
                        loadLimit:tableLimits.updateLimit,
                        limitedVisibility:true,
                        visibleLimit:tableLimits.visibleLimit,
                        parent:parent,
                        contentSelector:'#'+utils.escapeJQID(deviceName)+' tbody',
                        content:reportTable({columns:columns, rows:rows.slice(0,tableLimits.updateLimit),
                            options:{tableId:deviceName}}),
                        contentSize:rows.length
                    });

                    $('#'+utils.escapeJQID(deviceName)+' tbody').scroll(function (e) {
                        var _self = this;
                        var left = $(_self).scrollLeft();
                        $(_self).parent().children("thead").scrollLeft($(_self).scrollLeft());
                    });


                    datapointsReportDetails[deviceName] = {
                        columns: columns,
                        rows: rows,
                        currentpointer:tableLimits.updateLimit,
                        tableRef:displayTableScroll
                    };
                };

                var _addEventsForDatapointsReport = function () {
                    $(".expandCollapse").click(function (event) {
                        var currentTarget = event.target;
                        var deviceFile = $(currentTarget).attr("data-dev-file");
                        var deviceName = $(currentTarget).attr("data-dev-name");
                        if ($(currentTarget).hasClass("expand")) {
                            var isPointDataAvailable = $(currentTarget).attr("data-pointDataAvailable");
                            if (isPointDataAvailable === 'false') {
                                $(currentTarget).parent().find(".spinBlue").css('display','inline-block');
                                baja.Ord.make(deviceFile).get({
                                    ok: function (result) {
                                        $.get(result.getReadUri(), function (data) {
                                            var columns = [
                                                {name: "", field: "indicator"},
                                                {
                                                    name: lex.get("sbc.offlineEngineering.finish.report.dataPoint.pointName"),
                                                    field: "pointName"
                                                },
                                                {
                                                    name: lex.get("sbc.offlineEngineering.finish.report.dataPoint.mediaPoint"),
                                                    field: "mediaType"
                                                },
                                                {
                                                    name: lex.get("sbc.offlineEngineering.finish.report.dataPoint.memoryAddr"),
                                                    field: "memoryAddr"
                                                }
                                            ]
                                            var dataPoints = [];
                                            $.each($(data).find("Point"), function (p, e) {
                                                //var selected = $(e).attr("selected");
                                                var status = $(e).attr("status");
                                                //if (selected === "1") {
                                                    var state = "";
                                                    if (status === "1") {
                                                        state = "ok";
                                                    } else if (status === "0") {
                                                        state = "fail";
                                                    }
                                                    dataPoints.push({
                                                        importStatusClass: state,
                                                        pointName: $(e).attr("pointName"),
                                                        mediaType: $(e).attr("dataType"),
                                                        memoryAddr: $(e).attr("memoryAddress")
                                                    });
                                                //}
                                            });
                                            _createDatapointsReport(deviceName, columns, dataPoints, $(currentTarget).parent().next(".detailedDeviceImportTable"));
                                            $(currentTarget).attr("data-pointDataAvailable", "true");
                                            $(currentTarget).removeClass("expand");
                                            $(currentTarget).addClass("collapse");
                                            $($(currentTarget).parent().next()).css({'display': 'block'});

                                            var $detailWrapper = $(currentTarget).closest(".detailsWrapper");
                                            $detailWrapper.addClass("vheight");
                                            var minHeight = parseInt($detailWrapper.css('min-height'));
                                            var contentWrapperHeight = $('.reportContentWrapper').height();
                                            var contentHeight = (minHeight + (_TD_HGHT * (dataPoints.length - 1)));

                                            if(deviceData.data.length == 1) {
                                                $detailWrapper.css('height','100%');
                                            }

                                            //magic# to avoid scrollbar height
                                            $detailWrapper.css('max-height', Math.floor(contentHeight) + 5);

                                            if ($(".expandCollapse.deviceReportRow.collapse").length == 1) {
                                                $("#openAll").removeClass("openAll");
                                                $("#openAll").addClass("closeAll");
                                                $("#openAll").find("#openAllLabel").html(lex.get("sbc.offlineEngineering.finish.report.dataPoint.closeAll"));
                                            }

                                            //hide the spinner
                                            $(currentTarget).parent().find(".spinBlue").css('display', 'none');

                                            //enable Expand All button if all the devices are expanded
                                            if(--expandAllDevices == 0) {
                                                $("#openAll").css('pointerEvents', 'auto');
                                            }

                                        });
                                    }
                                });
                            } else {
                                $(currentTarget).removeClass("expand");
                                $(currentTarget).addClass("collapse");
                                $($(currentTarget).parent().next()).css({'display': 'block'});

                                var $detailWrapper = $(currentTarget).closest(".detailsWrapper");
                                $detailWrapper.addClass("vheight");
                                var minHeight = parseInt($detailWrapper.css('min-height'));
                                var contentWrapperHeight = $('.reportContentWrapper').height();
                                var contentHeight = (minHeight + (_TD_HGHT * (datapointsReportDetails[deviceName].rows.length - 1)));

                                if(deviceData.data.length == 1) {
                                        $detailWrapper.css('height','100%');
                                } else {
                                    //to undo height:auto
                                    $detailWrapper.css('height','');
                                }

                                //magic# to avoid scrollbar height
                                $detailWrapper.css('max-height', Math.floor(contentHeight) + 5);

                                //enable Expand All button if all the devices are expanded
                                if(--expandAllDevices == 0) {
                                    $("#openAll").css('pointerEvents', 'auto');
                                }

                            }
                        } else if ($(currentTarget).hasClass("collapse")) {
                            $($(currentTarget).parent().next()).css({'display': 'none'});
                            $(currentTarget).removeClass("collapse");
                            $(currentTarget).addClass("expand");
                            $(currentTarget).closest(".detailsWrapper").removeClass("vheight");
                            $(currentTarget).closest(".detailsWrapper").css('height', 'auto');
                        }

                        if ($(".expandCollapse.deviceReportRow.collapse").length == 1) {
                            $("#openAll").removeClass("openAll");
                            $("#openAll").addClass("closeAll");
                            $("#openAll").find("#openAllLabel").html(lex.get("sbc.offlineEngineering.finish.report.dataPoint.closeAll"));
                        } else if ($(".expandCollapse.deviceReportRow.collapse").length == 0) {
                            $("#openAll").removeClass("closeAll");
                            $("#openAll").addClass("openAll");
                            $("#openAll").find("#openAllLabel").html(lex.get("sbc.offlineEngineering.finish.report.dataPoint.openAll"));
                        }
                    });

                    $(".close").click(function (event) {
                        if(reportPopupDialog) {
                            reportPopupContent = "";
                            deviceData = {data: []};
                            reportPopupDialog.close();
                        }
                    });

                    $("#openAll").click(function (event) {
                        var target = event.target;

                        //Note: Do not allow expand/collapse all while its in progress already.
                        if(expandAllDevices > 0) {
                            return;
                        }

                        if($(target).hasClass("openAll")) {
                            //disable expand all untill the current operation is done.
                            expandAllDevices = $(".reportContentWrapper .expandCollapse.expand").length;
                            $(".reportContentWrapper .expandCollapse.expand").click();
                            $(target).removeClass("openAll");
                            $(target).addClass("closeAll");
                            $(target).find("#openAllLabel").html(lex.get("sbc.offlineEngineering.finish.report.dataPoint.closeAll"));
                        } else if($(target).hasClass("closeAll")) {
                            $(".reportContentWrapper .expandCollapse.collapse").click();
                            $(target).removeClass("closeAll");
                            $(target).addClass("openAll");
                            $(target).find("#openAllLabel").html(lex.get("sbc.offlineEngineering.finish.report.dataPoint.openAll"));
                        }
                    });
                };

                var _createDatapointsReportDialog = function () {
                    return dialogs.make({
                        content: reportPopupContent,
                        buttons: [
                            {
                                name: "print",
                                displayName: lex.get("sbc.offlineEngineering.finish.button.print"),
                                handler: function (dialog) {
                                    reportPopupContent = "";
                                    deviceData = {data: []};
                                    dialog.close();
                                }
                            }
                        ]
                    });
                };


                var _displayDatapointsImportStatus = function(event){
                    var systemOrd = devicePath + "bql:select deviceName, selectedDataPointsFile, numberOfSelectedDataPoints, numberOfCompletedDataPoints from sbcOfflineEngineering:SbcOfflineDevice where dataPointsSelected=true";
                    baja.Ord.make(systemOrd)
                        .get({
                            ok: function (result) {
                            },
                            cursor: {
                                each: function () {
                                    var deviceCursor = this;
                                    var deviceName = deviceCursor.get("deviceName") ? deviceCursor.get("deviceName") : "";
                                    var deviceFile = deviceCursor.get("selectedDataPointsFile") ? deviceCursor.get("selectedDataPointsFile") : "";
                                    var noOfSelectedDataPoints = deviceCursor.get("numberOfSelectedDataPoints") ? deviceCursor.get("numberOfSelectedDataPoints") : "";
                                    var noOfCompletedDataPoints = deviceCursor.get("numberOfCompletedDataPoints") ? deviceCursor.get("numberOfCompletedDataPoints") : "";

                                    deviceData.data.push({
                                        deviceName: deviceName,
                                        deviceFile: deviceFile.$ord,
                                        noOfSelectedDataPoints: noOfSelectedDataPoints,
                                        noOfCompletedItems: noOfCompletedDataPoints,
                                        noOfFailedItems: (noOfSelectedDataPoints - noOfCompletedDataPoints)
                                    });
                                },
                                after: function () {
                                    var data = deviceData.data;
                                    if (data) {
                                        reportPopupDialog = dialogs.make({
                                            content: reportTemplate({
                                                openAll: lex.get("sbc.offlineEngineering.finish.report.dataPoint.openAll"),
                                                reportTitle:lex.get("sbc.offlineEngineering.finish.report.dataPoint"),
                                                success:lex.get("sbc.offlineEngineering.finish.report.dataPoint.importSuccess"),
                                                successCount:sbcService.getDataPoints().getCompletedDataPointsCount(),
                                                failure:lex.get("sbc.offlineEngineering.finish.report.dataPoint.importFail"),
                                                failureCount: (sbcService.getDataPoints().getSelectedDataPointsCount() - sbcService.getDataPoints().getCompletedDataPointsCount()),
                                                devices:data}),
                                            buttons: [
                                                {
                                                    name: "print",
                                                    displayName: lex.get("sbc.offlineEngineering.finish.button.print"),
                                                    handler: function (dialog) {
                                                        reportPopupContent = "";
                                                        deviceData = {data: []};
                                                        dialog.close();
                                                    }
                                                }
                                            ]
                                        });
                                        reportPopupDialog.show({content: reportPopupContent});
                                        $(".js-dialog-button").prop("disabled", true);

                                        $("#detailedReport").parents(".js-dialog").css({
                                            'width': '80%',
                                            'height': '100%',
                                            'background-color': '#eaebeb'
                                        });
                                        var jsDialogButtonContent = $("#detailedReport").parents(".js-dialog-content-wrapper")
                                            .css({'height': '100%','overflow':'hidden'}).next();
                                        $(jsDialogButtonContent).css({ 'background-color': '#eaebeb', 'float':'right'});
                                        $("#detailedReport").parent(".js-dialog-content").css({'height': '98%','box-sizing':'border-box'});

                                        _addEventsForDatapointsReport();


/*                                        $(".close").click(function (event) {
                                            if(reportPopupDialog) {
                                                reportPopupContent = "";
                                                deviceData = {data: []};
                                                reportPopupDialog.close();
                                            }
                                        });

                                        $(".expandCollapse").click(function (event) {
                                            var currentTarget = event.target;
                                            var deviceFile = $(currentTarget).attr("data-dev-file");
                                            var deviceName = $(currentTarget).attr("data-dev-name");
                                            if ($(currentTarget).hasClass("expand")) {
                                                var isPointDataAvailable = $(currentTarget).attr("data-pointDataAvailable");
                                                if (isPointDataAvailable === 'false') {
                                                    baja.Ord.make(deviceFile).get({
                                                        ok: function (result) {
                                                            $.get(result.getReadUri(), function (data) {
                                                                var columns = [
                                                                    {name: "", field: "indicator"},
                                                                    {
                                                                        name: lex.get("sbc.offlineEngineering.finish.report.dataPoint.pointName"),
                                                                        field: "pointName"
                                                                    },
                                                                    {
                                                                        name: lex.get("sbc.offlineEngineering.finish.report.dataPoint.mediaPoint"),
                                                                        field: "mediaType"
                                                                    },
                                                                    {
                                                                        name: lex.get("sbc.offlineEngineering.finish.report.dataPoint.memoryAddr"),
                                                                        field: "memoryAddr"
                                                                    }
                                                                ]
                                                                var dataPoints = [];
                                                                $.each($(data).find("Point"), function (p, e) {
                                                                    var selected = $(e).attr("selected");
                                                                    var status = $(e).attr("status");
                                                                    //if (selected === "1") {
                                                                    var state = "";
                                                                    if (status === "1") {
                                                                        state = "ok";
                                                                    } else if (status === "0") {
                                                                        state = "fail";
                                                                    }
                                                                    dataPoints.push({
                                                                        importStatusClass: state,
                                                                        pointName: $(e).attr("pointName"),
                                                                        mediaType: $(e).attr("dataType"),
                                                                        memoryAddr: $(e).attr("memoryAddress")
                                                                    });
                                                                    //}
                                                                });
                                                                _createDatapointsReport(deviceName, columns, dataPoints, $(currentTarget).parent().next(".detailedDeviceImportTable"));
                                                                $(currentTarget).attr("data-pointDataAvailable", "true");
                                                                $(currentTarget).removeClass("expand");
                                                                $(currentTarget).addClass("collapse");
                                                                $($(currentTarget).parent().next()).css({'display': 'block'});
                                                                $(currentTarget).closest(".detailsWrapper").addClass("vheight");
                                                            });
                                                        }
                                                    });
                                                } else {
                                                    $(currentTarget).removeClass("expand");
                                                    $(currentTarget).addClass("collapse");
                                                    $($(currentTarget).parent().next()).css({'display': 'block'});
                                                    $(currentTarget).closest(".detailsWrapper").addClass("vheight");
                                                }
                                            } else if ($(currentTarget).hasClass("collapse")) {
                                                $($(currentTarget).parent().next()).css({'display': 'none'});
                                                $(currentTarget).removeClass("collapse");
                                                $(currentTarget).addClass("expand");
                                                $(currentTarget).closest(".detailsWrapper").removeClass("vheight");
                                            }
                                        });
                                        */
                                    }
                                },
                                limit: 50,
                                offset: 0
                            }
                        });
                };




                /*Alarm List Import Report code starts from here */

                var alarmListReportDetails = {};

                var _getAlarmListsTRContent = function (deviceName, start, end) {
                    var deviceDetails = alarmListReportDetails[deviceName];
                    var l1 = (start  != undefined) ? start : deviceDetails.currentpointer,
                        l2 = (end != undefined) ? end : (deviceDetails.currentpointer+tableLimits.updateLimit);
                    deviceDetails.currentpointer = l2;

                    var e = reportTableRow({
                        columns: deviceDetails.columns,
                        rows: deviceDetails.rows.slice(l1, l2)
                    });
                    return e.trim();
                };

                var _createAlarmListsReport = function (deviceName, columns, rows, parent) {

                    var displayTableScroll = nscroll({ getContent:function(deviceName){
                            return function(start, end){return _getAlarmListsTRContent(deviceName, start, end);}}(deviceName),
                        onLoad: null,
                        onUnload: null,
                        updateLimit: 50,
                        loadLimit:tableLimits.updateLimit,
                        limitedVisibility:true,
                        visibleLimit:tableLimits.visibleLimit,
                        parent:parent,
                        contentSelector:'#'+utils.escapeJQID(deviceName)+' tbody',
                        content:reportTable({columns:columns, rows:rows.slice(0,tableLimits.updateLimit),
                            options:{tableId:deviceName}}),
                        contentSize:rows.length
                    });

                    $('#'+utils.escapeJQID(deviceName)+' tbody').scroll(function (e) {
                        var _self = this;
                        var left = $(_self).scrollLeft();
                        $(_self).parent().children("thead").scrollLeft($(_self).scrollLeft());
                    });


                    alarmListReportDetails[deviceName] = {
                        columns: columns,
                        rows: rows,
                        currentpointer:tableLimits.updateLimit,
                        tableRef:displayTableScroll
                    };
                };

                 var _addEventsForAlarmListsReport = function () {
                    $(".expandCollapse").click(function (event) {
                        var currentTarget = event.target;
                        var deviceFile = $(currentTarget).attr("data-dev-file");
                        var deviceName = $(currentTarget).attr("data-dev-name");
                        if ($(currentTarget).hasClass("expand")) {
                            var isPointDataAvailable = $(currentTarget).attr("data-pointDataAvailable");
                            if (isPointDataAvailable === 'false') {
                                $(currentTarget).parent().find(".spinBlue").css('display','inline-block');
                                baja.Ord.make(deviceFile).get({
                                    ok: function (result) {
                                        $.get(result.getReadUri(), function (data) {
                                            var columns = [
                                                {name: "", field: "indicator"},
                                                {
                                                    name: lex.get("sbc.offlineEngineering.finish.report.alarmList.alarmList"),
                                                    field: "alarmList"
                                                },
                                                {
                                                    name: lex.get("sbc.offlineEngineering.finish.report.alarmList.solution"),
                                                    field: "solution"
                                                },
                                                {
                                                    name: lex.get("sbc.offlineEngineering.finish.report.alarmList.project"),
                                                    field: "project"
                                                }
                                            ]
                                            var alarmListData = [];
                                            var solutionName,projectName, alarmListName;
                                            var solutionNode, projectNode, alarmListNode;
                                            var solutions = $(data).find('Solution');
                                            if(solutions.length>0){
                                                var i, j, k;
                                                for(i=0;i<solutions.length; i++) {
                                                    solutionNode = $(solutions[i]);
                                                    solutionName = $(solutionNode).attr("solutionName");
                                                    var projects = $(solutionNode).find('Project');
                                                    for(j=0;j<projects.length;j++){
                                                        projectNode = $(projects[j]);
                                                        projectName = $(projectNode).attr("projectName");
                                                        var alarmlists = $(projectNode).find('AlarmList');
                                                            for(k=0;k<alarmlists.length;k++){
                                                                alarmListNode = $(alarmlists[k]);
                                                                var selected = $(alarmListNode).attr("selected");
                                                                var status = $(alarmListNode).attr("status");
                                                                 var state = "";
                                                                if(selected==="1"){
                                                                    $(alarmListNode).attr("symbolName");
                                                                    if (status === "1") {
                                                                        state = "ok";
                                                                    } else{
                                                                        state = "fail";
                                                                    }
                                                                    alarmListData.push({
                                                                        importStatusClass: state,
                                                                        alarmList: $(alarmListNode).attr("symbolName"),
                                                                        solution: solutionName,
                                                                        project: projectName
                                                                    });
                                                                }
                                                            }
                                                    }
                                                }
                                            }
                                            _createAlarmListsReport(deviceName, columns, alarmListData, $(currentTarget).parent().next(".detailedDeviceImportTable"));
                                            $(currentTarget).attr("data-pointDataAvailable", "true");
                                            $(currentTarget).removeClass("expand");
                                            $(currentTarget).addClass("collapse");
                                            $($(currentTarget).parent().next()).css({'display': 'block'});

                                            var $detailWrapper = $(currentTarget).closest(".detailsWrapper");
                                            $detailWrapper.addClass("vheight");
                                            var minHeight = parseInt($detailWrapper.css('min-height'));
                                            var contentWrapperHeight = $('.reportContentWrapper').height();
                                            var contentHeight = (minHeight + (_TD_HGHT * (alarmListData.length - 1)));

                                            if(deviceData.data.length == 1) {
                                                $detailWrapper.css('height','100%');
                                            }

                                            $detailWrapper.css('max-height', Math.floor(contentHeight) + 5);

                                            if ($(".expandCollapse.deviceReportRow.collapse").length == 1) {
                                                $("#openAll").removeClass("openAll");
                                                $("#openAll").addClass("closeAll");
                                                $("#openAll").find("#openAllLabel").html(lex.get("sbc.offlineEngineering.finish.report.dataPoint.closeAll"));
                                            }

                                            //hide the spinner
                                            $(currentTarget).parent().find(".spinBlue").css('display', 'none');

                                        });
                                    }
                                });
                            } else {
                                $(currentTarget).removeClass("expand");
                                $(currentTarget).addClass("collapse");
                                $($(currentTarget).parent().next()).css({'display': 'block'});

                                var $detailWrapper = $(currentTarget).closest(".detailsWrapper");
                                $detailWrapper.addClass("vheight");
                                var minHeight = parseInt($detailWrapper.css('min-height'));
                                var contentWrapperHeight = $('.reportContentWrapper').height();
                                var contentHeight = (minHeight + (_TD_HGHT * (alarmListReportDetails[deviceName].rows.length - 1)));

                                if(deviceData.data.length == 1) {
                                    $detailWrapper.css('height','100%');
                                } else {
                                    //to undo height:auto
                                    $detailWrapper.css('height','');
                                }
                                //magic# to avoid scrollbar height
                                $detailWrapper.css('max-height', Math.floor(contentHeight) + 5);

                            }
                        } else if ($(currentTarget).hasClass("collapse")) {
                            $($(currentTarget).parent().next()).css({'display': 'none'});
                            $(currentTarget).removeClass("collapse");
                            $(currentTarget).addClass("expand");
                            $(currentTarget).closest(".detailsWrapper").removeClass("vheight");
                            $(currentTarget).closest(".detailsWrapper").css("height",'auto');
                        }

                        if ($(".expandCollapse.deviceReportRow.collapse").length == 1) {
                            $("#openAll").removeClass("openAll");
                            $("#openAll").addClass("closeAll");
                            $("#openAll").find("#openAllLabel").html(lex.get("sbc.offlineEngineering.finish.report.dataPoint.closeAll"));
                        } else if ($(".expandCollapse.deviceReportRow.collapse").length == 0) {
                            $("#openAll").removeClass("closeAll");
                            $("#openAll").addClass("openAll");
                            $("#openAll").find("#openAllLabel").html(lex.get("sbc.offlineEngineering.finish.report.dataPoint.openAll"));
                        }
                    });

                    $(".close").click(function (event) {
                        if(reportPopupDialog) {
                            reportPopupContent = "";
                            deviceData = {data: []};
                            reportPopupDialog.close();
                        }
                    });

                    $("#openAll").click(function (event) {
                        var target = event.target;
                        if($(target).hasClass("openAll")) {
                            $(".reportContentWrapper .expandCollapse.expand").click();
                            $(target).removeClass("openAll");
                            $(target).addClass("closeAll");
                            $(target).find("#openAllLabel").html(lex.get("sbc.offlineEngineering.finish.report.dataPoint.closeAll"));
                        } else if($(target).hasClass("closeAll")) {
                            $(".reportContentWrapper .expandCollapse.collapse").click();
                            $(target).removeClass("closeAll");
                            $(target).addClass("openAll");
                            $(target).find("#openAllLabel").html(lex.get("sbc.offlineEngineering.finish.report.dataPoint.openAll"));
                        }
                    });
                };


                var _displayAlarmListsImportStatus = function(event){
                    var systemOrd = devicePath + "bql:select deviceName, alarmListFile, numberOfSelectedAlarmLists, numberOfCompletedAlarmLists from sbcOfflineEngineering:SbcOfflineDevice where alarmListSelected=true";
                    baja.Ord.make(systemOrd)
                        .get({
                            ok: function (result) {
                            },
                            cursor: {
                                each: function () {
                                    var deviceCursor = this;
                                    var deviceName = deviceCursor.get("deviceName") ? deviceCursor.get("deviceName") : "";
                                    var deviceFile = deviceCursor.get("alarmListFile") ? deviceCursor.get("alarmListFile") : "";
                                    var noOfSelectedAlarmLists = deviceCursor.get("numberOfSelectedAlarmLists") ? deviceCursor.get("numberOfSelectedAlarmLists") : "";
                                    var noOfCompletedAlarmLists = deviceCursor.get("numberOfCompletedAlarmLists") ? deviceCursor.get("numberOfCompletedAlarmLists") : "";

                                    deviceData.data.push({
                                        deviceName: deviceName,
                                        deviceFile: deviceFile.$ord,
                                        noOfSelectedAlarmLists: noOfSelectedAlarmLists,
                                        noOfCompletedItems: noOfCompletedAlarmLists,
                                        noOfFailedItems: (noOfSelectedAlarmLists - noOfCompletedAlarmLists)
                                    });
                                },
                                after: function () {
                                    var data = deviceData.data;
                                    if (data) {
                                        reportPopupDialog = dialogs.make({
                                            content: reportTemplate({openAll: lex.get("sbc.offlineEngineering.finish.report.dataPoint.openAll"),
                                                reportTitle:lex.get("sbc.offlineEngineering.finish.report.alarmList"),
                                                success:lex.get("sbc.offlineEngineering.finish.report.dataPoint.importSuccess"),
                                                successCount:sbcService.getAlarms().getCompletedAlarmListCount(),
                                                failure:lex.get("sbc.offlineEngineering.finish.report.dataPoint.importFail"),
                                                failureCount: (sbcService.getAlarms().getSelectedAlarmListCount() - sbcService.getAlarms().getCompletedAlarmListCount()),
                                                devices:data}),
                                            buttons: [
                                                {
                                                    name: "print",
                                                    displayName: lex.get("sbc.offlineEngineering.finish.button.print"),
                                                    handler: function (dialog) {
                                                        reportPopupContent = "";
                                                        deviceData = {data: []};
                                                        dialog.close();
                                                    }
                                                }
                                            ]
                                        });
                                        reportPopupDialog.show({content: reportPopupContent});
                                        $(".js-dialog-button").prop("disabled", true);

                                        $("#detailedReport").parents(".js-dialog").css({
                                            'width': '80%',
                                            'height': '100%',
                                            'background-color': '#eaebeb'
                                        });
                                        var jsDialogButtonContent = $("#detailedReport").parents(".js-dialog-content-wrapper")
                                            .css({'height': '100%','overflow':'hidden'}).next();
                                        $(jsDialogButtonContent).css({ 'background-color': '#eaebeb', 'float':'right'});
                                        $("#detailedReport").parent(".js-dialog-content").css({'height': '98%','box-sizing':'border-box'});

                                        _addEventsForAlarmListsReport();
                                    }
                                },
                                limit: 50,
                                offset: 0
                            }
                        });
                };

                /*Alarm List Import report code ends here */

                /*HDLog Import report code ends here */
				var hdLogsReportDetails = {};

                var _getHdLogsTRContent = function (deviceName, start, end) {
                    var deviceDetails = hdLogsReportDetails[deviceName];
                    var l1 = (start  != undefined) ? start : deviceDetails.currentpointer,
                        l2 = (end != undefined) ? end : (deviceDetails.currentpointer+tableLimits.updateLimit);
                    deviceDetails.currentpointer = l2;

                    var e = reportTableRow({
                        columns: deviceDetails.columns,
                        rows: deviceDetails.rows.slice(l1, l2)
                    });
                    return e.trim();
                };

                var _createHdLogsReport = function (deviceName, columns, rows, parent) {

                    var displayTableScroll = nscroll({ getContent:function(deviceName){
                            return function(start, end){return _getHdLogsTRContent(deviceName, start, end);}}(deviceName),
                        onLoad: null,
                        onUnload: null,
                        updateLimit: 50,
                        loadLimit:tableLimits.updateLimit,
                        limitedVisibility:true,
                        visibleLimit:tableLimits.visibleLimit,
                        parent:parent,
                        contentSelector:'#'+utils.escapeJQID(deviceName)+' tbody',
                        content:reportTable({columns:columns, rows:rows.slice(0,tableLimits.updateLimit),
                            options:{tableId:deviceName}}),
                        contentSize:rows.length
                    });

                    $('#'+utils.escapeJQID(deviceName)+' tbody').scroll(function (e) {
                        var _self = this;
                        var left = $(_self).scrollLeft();
                        $(_self).parent().children("thead").scrollLeft($(_self).scrollLeft());
                    });


                    hdLogsReportDetails[deviceName] = {
                        columns: columns,
                        rows: rows,
                        currentpointer:tableLimits.updateLimit,
                        tableRef:displayTableScroll
                    };
                };

                var _addEventsForHdLogsReport = function () {
                    $(".expandCollapse").click(function (event) {
                        var currentTarget = event.target;
                        var deviceFile = $(currentTarget).attr("data-dev-file");
                        var deviceName = $(currentTarget).attr("data-dev-name");
                        if ($(currentTarget).hasClass("expand")) {
                            var isPointDataAvailable = $(currentTarget).attr("data-pointDataAvailable");
                            if (isPointDataAvailable === 'false') {
                                $(currentTarget).parent().find(".spinBlue").css('display','inline-block');
                                baja.Ord.make(deviceFile).get({
                                    ok: function (result) {
                                        $.get(result.getReadUri(), function (data) {
                                            var columns = [
                                                {name: "", field: "indicator"},
                                                {
                                                    name: lex.get("sbc.offlineEngineering.finish.report.hdlog.hdlogname"),
                                                    field: "hdLogName"
                                                }
                                            ]
                                            var hdLogs = [];
                                            $.each($(data).find("HDLog"), function (p, e) {
                                                //var selected = $(e).attr("selected");
                                                var status = $(e).attr("status");
                                                var selected = $(e).attr("selected");
                                                if (selected === "1") {
                                                    var state = "";
                                                        if (status === "1") {
                                                            state = "ok";
                                                        } else  {
                                                            state = "fail";
                                                        }

                                                    hdLogs.push({
                                                        importStatusClass: state,
                                                        hdLogName: $(e).attr("logname")
                                                    });
                                                }
                                            });
                                            _createHdLogsReport(deviceName, columns, hdLogs, $(currentTarget).parent().next(".detailedDeviceImportTable"));
                                            $(currentTarget).attr("data-pointDataAvailable", "true");
                                            $(currentTarget).removeClass("expand");
                                            $(currentTarget).addClass("collapse");
                                            $($(currentTarget).parent().next()).css({'display': 'block'});
                                            $($(currentTarget).parent().next()).addClass("detailedReportHdLogRow");

                                            var $detailWrapper = $(currentTarget).closest(".detailsWrapper");
                                            $detailWrapper.addClass("vheight");
                                            var minHeight = parseInt($detailWrapper.css('min-height'));
                                            var contentWrapperHeight = $('.reportContentWrapper').height();
                                            var contentHeight = (minHeight + (_TD_HGHT * (hdLogs.length - 1)));

                                            if(deviceData.data.length == 1) {
                                                $detailWrapper.css('height','100%');
                                            }

                                            //magic# to avoid scrollbar height
                                            $detailWrapper.css('max-height', Math.floor(contentHeight) + 5);

                                            if ($(".expandCollapse.deviceReportRow.collapse").length == 1) {
                                                $("#openAll").removeClass("openAll");
                                                $("#openAll").addClass("closeAll");
                                                $("#openAll").find("#openAllLabel").html(lex.get("sbc.offlineEngineering.finish.report.dataPoint.closeAll"));
                                            }

                                            //hide the spinner
                                            $(currentTarget).parent().find(".spinBlue").css('display', 'none');

                                        });
                                    }
                                });
                            } else {
                                $(currentTarget).removeClass("expand");
                                $(currentTarget).addClass("collapse");
                                $($(currentTarget).parent().next()).css({'display': 'block'});

                                var $detailWrapper = $(currentTarget).closest(".detailsWrapper");
                                $detailWrapper.addClass("vheight");
                                var minHeight = parseInt($detailWrapper.css('min-height'));
                                var contentWrapperHeight = $('.reportContentWrapper').height();
                                var contentHeight = (minHeight + (_TD_HGHT * (hdLogsReportDetails[deviceName].rows.length - 1)));

                                if(deviceData.data.length == 1) {
                                    $detailWrapper.css('height','100%');
                                } else {
                                    //to undo height:auto
                                    $detailWrapper.css('height','');
                                }

                                //magic# to avoid scrollbar height
                                $detailWrapper.css('max-height', Math.floor(contentHeight) + 5);

                            }
                        } else if ($(currentTarget).hasClass("collapse")) {
                            $($(currentTarget).parent().next()).css({'display': 'none'});
                            $($(currentTarget).parent().next()).addClass("detailedReportHdLogRow");
                            $(currentTarget).removeClass("collapse");
                            $(currentTarget).addClass("expand");
                            $(currentTarget).closest(".detailsWrapper").removeClass("vheight");
                            $(currentTarget).closest(".detailsWrapper").css('height','auto');
                        }

                        if ($(".expandCollapse.deviceReportRow.collapse").length == 1) {
                            $("#openAll").removeClass("openAll");
                            $("#openAll").addClass("closeAll");
                            $("#openAll").find("#openAllLabel").html(lex.get("sbc.offlineEngineering.finish.report.dataPoint.closeAll"));
                        } else if ($(".expandCollapse.deviceReportRow.collapse").length == 0) {
                            $("#openAll").removeClass("closeAll");
                            $("#openAll").addClass("openAll");
                            $("#openAll").find("#openAllLabel").html(lex.get("sbc.offlineEngineering.finish.report.dataPoint.openAll"));
                        }
                    });

                    $(".close").click(function (event) {
                        if(reportPopupDialog) {
                            reportPopupContent = "";
                            deviceData = {data: []};
                            reportPopupDialog.close();
                        }
                    });

                    $("#openAll").click(function (event) {
                        var target = event.target;
                        if($(target).hasClass("openAll")) {
                            $(".reportContentWrapper .expandCollapse.expand").click();
                            $(target).removeClass("openAll");
                            $(target).addClass("closeAll");
                            $(target).find("#openAllLabel").html(lex.get("sbc.offlineEngineering.finish.report.dataPoint.closeAll"));
                        } else if($(target).hasClass("closeAll")) {
                            $(".reportContentWrapper .expandCollapse.collapse").click();
                            $(target).removeClass("closeAll");
                            $(target).addClass("openAll");
                            $(target).find("#openAllLabel").html(lex.get("sbc.offlineEngineering.finish.report.dataPoint.openAll"));
                        }
                    });
                };



                var _displayHdLogsImportStatus = function(event){
                    var systemOrd = devicePath + "bql:select deviceName, hDLogListFile, numberOfSelectedHDLogs, numberOfCompletedHDLogs from sbcOfflineEngineering:SbcOfflineDevice where hdLogsSelected=true";
                    baja.Ord.make(systemOrd)
                        .get({
                            ok: function (result) {
                            },
                            cursor: {
                                each: function () {
                                    var deviceCursor = this;
                                    var deviceName = deviceCursor.get("deviceName") ? deviceCursor.get("deviceName") : "";
                                    var deviceFile = deviceCursor.get("hDLogListFile") ? deviceCursor.get("hDLogListFile") : "";
                                    var noOfSelectedHdLogs = deviceCursor.get("numberOfSelectedHDLogs") ? deviceCursor.get("numberOfSelectedHDLogs") : "";
                                    var noOfCompletedHdLogs = deviceCursor.get("numberOfCompletedHDLogs") ? deviceCursor.get("numberOfCompletedHDLogs") : "";

                                    deviceData.data.push({
                                        deviceName: deviceName,
                                        deviceFile: deviceFile.$ord,
                                        numberOfSelectedHdLogs: noOfSelectedHdLogs,
                                        noOfCompletedItems: noOfCompletedHdLogs,
                                        noOfFailedItems: (noOfSelectedHdLogs - noOfCompletedHdLogs)
                                    });
                                },
                                after: function () {
                                    var data = deviceData.data;
                                    if (data) {
                                        reportPopupDialog = dialogs.make({
                                            content: reportTemplate({openAll: lex.get("sbc.offlineEngineering.finish.report.dataPoint.openAll"),
                                                reportTitle:lex.get("sbc.offlineEngineering.finish.report.hdlogs"),
                                                success:lex.get("sbc.offlineEngineering.finish.report.dataPoint.importSuccess"),
                                                successCount:sbcService.getHdLogs().getCompletedHDLogsCount(),
                                                failure:lex.get("sbc.offlineEngineering.finish.report.dataPoint.importFail"),
                                                failureCount: (sbcService.getHdLogs().getSelectedHDLogsCount() - sbcService.getHdLogs().getCompletedHDLogsCount()),
                                                devices:data}),
                                            buttons: [
                                                {
                                                    name: "print",
                                                    displayName: lex.get("sbc.offlineEngineering.finish.button.print"),
                                                    handler: function (dialog) {
                                                        reportPopupContent = "";
                                                        deviceData = {data: []};
                                                        dialog.close();
                                                    }
                                                }
                                            ]
                                        });
                                        reportPopupDialog.show({content: reportPopupContent});
                                        $(".js-dialog-button").prop("disabled", true);

                                        $("#detailedReport").parents(".js-dialog").css({
                                            'width': '80%',
                                            'height': '100%',
                                            'background-color': '#eaebeb'
                                        });
                                        var jsDialogButtonContent = $("#detailedReport").parents(".js-dialog-content-wrapper")
                                            .css({'height': '100%','overflow':'hidden'}).next();
                                        $(jsDialogButtonContent).css({ 'background-color': '#eaebeb', 'float':'right'});
                                        $("#detailedReport").parent(".js-dialog-content").css({'height': '98%','box-sizing':'border-box'});

                                        _addEventsForHdLogsReport();
                                    }
                                },
                                limit: 50,
                                offset: 0
                            }
                        });
                };
                /*HDLog Import report code ends here */

                return {

                    /**
                     *
                     * @param element: dom element the view will be added to
                     * @param params: javascript object with widget parameters required to load the view within the widget
                     */

                    loadView: function (element, params) {
                        _resetData();
                        $element = $(element);
                        $widgetContainer = (params.widgetContainer) ? $(params.widgetContainer) : (element).closest(".sbcOfflineEngineering-wizard");
                        selectedModules = params.selectedModules;
                        importStatus = params.selectedModules;
                        sbcService = params.component;

                        $(element).append(template({
                            defaultActionMsg: lex.get("sbc.offlineEngineering.import.complete"),
                            datapoints: lex.get("sbc.offlineEngineering.select.datapoints"),
                            hdlogs: lex.get("sbc.offlineEngineering.select.hdlogs"),
                            alarms: lex.get("sbc.offlineEngineering.select.alarms"),
                            detailBtn: lex.get("sbc.offlineEngineering.detail"),
                            selectedAlarms: true,
                            selectedHdlogs: true
                        }));

                        $widgetContainer.find("#sbcOfflineEngineering-cancelBttn").css("display", "none");
                        $widgetContainer.find("#sbcOfflineEngineering-backBttn").css("display", "none");
                        $widgetContainer.find("#sbcOfflineEngineering-nextBttn").css("display", "inline-block");

                        var dataPointsDetailsButton = $widgetContainer.find("#sbcOfflineEngineering-dpDetailBtn");
                        if(selectedModules.datapoints) {
                            dataPointsDetailsButton.prop('disabled',false);
                            dataPointsDetailsButton.click(_displayDatapointsImportStatus);
                        }

                        var alarmListsDetailsButton = $widgetContainer.find("#sbcOfflineEngineering-alDetailBtn");
                        if(selectedModules.alarms) {
                            alarmListsDetailsButton.prop('disabled',false);
                            alarmListsDetailsButton.click(_displayAlarmListsImportStatus);
                        }

                        var hdLogsDetailsButton = $widgetContainer.find("#sbcOfflineEngineering-hdlDetailBtn");
                        if(selectedModules.hdlogs) {
                            hdLogsDetailsButton.prop('disabled',false);
                            hdLogsDetailsButton.click(_displayHdLogsImportStatus);
                        }

                        var datapointsContainer = sbcService.getDataPoints(), alarmsContainer = sbcService.getAlarms(), hdlogsContainer = sbcService.getHdLogs(), dpIconPath, alarmsIconPath, hdLogsIconPath;

                        if (selectedModules.datapoints) {
                            if (datapointsContainer.getCompletedDataPointsCount() == datapointsContainer.getSelectedDataPointsCount()) {
                                dpIconPath = "/module/sbcOfflineEngineering/rc/images/data_points_blue_ok.svg";
                            } else {
                                dpIconPath = "/module/sbcOfflineEngineering/rc/images/data_points_blue_error_red.svg";
                            }
                        } else {
                            dpIconPath = "/module/sbcOfflineEngineering/rc/images/scan_datapoints_default.svg";
                        }

                        if (selectedModules.alarms) {
                            if (alarmsContainer.getCompletedAlarmListCount() == alarmsContainer.getSelectedAlarmListCount()) {
                                alarmsIconPath = "/module/sbcOfflineEngineering/rc/images/alarms_blue_ok.svg";
                            } else {
                                alarmsIconPath = "/module/sbcOfflineEngineering/rc/images/alarms_blue_error_red.svg";
                            }
                        } else {
                            alarmsIconPath = "/module/sbcOfflineEngineering/rc/images/scan_alarms_default.svg";
                        }

                        if (selectedModules.hdlogs) {
                            if (hdlogsContainer.getCompletedHDLogsCount() == hdlogsContainer.getSelectedHDLogsCount()) {
                                hdLogsIconPath = "/module/sbcOfflineEngineering/rc/images/hd_logs_blue_ok.svg";
                            } else {
                                hdLogsIconPath = "/module/sbcOfflineEngineering/rc/images/hd_logs_error_red.svg";
                            }
                        } else {
                            hdLogsIconPath = "/module/sbcOfflineEngineering/rc/images/scan_hd_logs_default.svg";
                        }
                        progress.prototype.buildAtFinish("#finish-datapoints-circle", "#finish-alarms-circle", "#finish-hdlogs-circle", dpIconPath, alarmsIconPath, hdLogsIconPath);
                        sbcService.remove("devicesNewNames");
                    },

                    /**
                     * data to be passed to widget to communicate between views
                     *
                     * @returns {json}: return key value pairs of data
                     */

                    getData: function () {
                        return {scannedModules: importList};
                    },

                    /**
                     * validate user input
                     *
                     * @returns {boolean}: return true/false based on validity of user input
                     */

                    validate: function () {
                        return new Promise(function (resolve) {
                            resolve(true);
                        });
                    }
                    ,

                    next: function (event, params) {
                        return new Promise(function (resolve) {
                            resolve();
                        });
                    },
                    params: {
                        vname: 'finishView',
                        id: 'finishView',
                        message: lex.get('sbc.offlineEngineering.scan'),
                        nextBttnMsg: lex.get('sbc.offlineEngineering.newImport'),
                        isCancelAvailable: true,
                        cancelBttnMsg: lex.get('sbc.offlineEngineering.cancel')
                    }
                };
            }
            ;

        return finishView;
    }
);