/**
 * * Copyright (c) 2018. Honeywell International, Inc. All Rights Reserved.
 */

define(['baja!',
        'jquery',
        'Promise',
        'underscore',
        'dialogs',
        'nmodule/sbcOfflineEngineering/rc/js/util/Progress',
        'hbs!nmodule/sbcOfflineEngineering/rc/hbs/importView',
        'lex!sbcOfflineEngineering',
        'nmodule/sbcOfflineEngineering/rc/js/lib/jquery-ui'
    ],
    function (baja,
              $,
              Promise,
              _,
              dialogs,
              progress,
              template,
              Lexicons) {

        'use strict';

        /**
         * Description of your widget.
         *
         * @class
         * @alias module:nmodule/sbcOfflineEngineering/rc/js/OfflineEngineeringimportView
         */
        var importView = new function () {

            var lex = Lexicons[0];
            var $element = null;
            var selectedModules = {};
            var importStatus = {};
            var importList = {datapoints: 0, alarms: 0, hdlogs: 0};
            var sbcService = null;
            var $widgetContainer = null;
            var isImportProcessCancelled=false;
            var cancelDialog = null;
            var promptDialog = null ;
            var promptIpDialog = null;
            var devicesNewNamesMainComp=new baja.Component();
            var devicesNewIpMainComp=new baja.Component();
            var deviceInfoPath = "file:^offline/engineering/system.bog|bog:|slot:|";

            var _updateBttnsOnImport = function () {
                $widgetContainer.find("#sbcOfflineEngineering-cancelBttn").prop("disabled", false);
                $widgetContainer.find("#sbcOfflineEngineering-backBttn").prop("disabled", true);
            };

            var _updateBttnsOnCancel = function () {
                $widgetContainer.find("#sbcOfflineEngineering-cancelBttn").prop("disabled", true);
                $widgetContainer.find("#sbcOfflineEngineering-backBttn").prop("disabled", false);
                $widgetContainer.find("#sbcOfflineEngineering-nextBttn").css("display","inline-block");
                $widgetContainer.find("#sbcOfflineEngineering-nextBttn").prop("disabled", false);
            };


            /**
             *_getDeviceData: get selected list of devices.
             * @param resolve
             * @param params
             *
             */

            var _getDeviceData = function () {
                return new Promise(function (resolve) {
                    var dataOrd = deviceInfoPath + "bql:select * from sbcOfflineEngineering:SbcOfflineDevice where dataPointsSelected = true or alarmListSelected = true or hdLogsSelected = true";
                    var deviceDetails;
                    try {
                        baja.Ord.make(dataOrd).get({
                            ok: function (result) {
                                //returns resolved ord target
                            },
                            cursor: {
                                each: function () {
                                    deviceDetails = this;
                                },
                                after: function () {
                                    try {
                                        var totalDevices = deviceDetails.$curData;
                                            deviceConflicts(resolve,totalDevices);
                                    } catch (e) {
                                        console.log(e);
                                    }
                                }
                            }
                        });

                    } catch (e) {
                        console.log(e);
                    }

                });
                }

            /**
             * Check for device name conflicts.
             * @param resolve
             * @param devices
             */
            var deviceConflicts= function (resolve, devices) {
                checkForSameName(resolve,devices, 0);
                //TO Do for IP address conflict
            }

            /**
             * Check if any other device has same name.
             * @param resolve
             * @param devices
             * @param index
             */

            var checkForSameName = function(resolve, devices, index){
                var comp = devices[index];
                var devicePath = "local:|foxs:|station:|slot:/Drivers/SBCIpNetwork|bql:select slotPath from sbc:SbcDevice where deviceName='" + comp.get("deviceName") + "' and ipAddress.ipAddress!='" + comp.get("ipAddress") + "' and deviceType!='Sub Station'";
                try {
                    var currentDevice = null;
                    baja.Ord.make(devicePath)
                        .get({
                            cursor: {
                                each: function(){
                                    currentDevice = this;
                                },
                                after: function(){
                                    if(index<devices.length){
                                        if(currentDevice!=null){
                                            _promptDialog(resolve, devices, index);
                                        }else{
                                            index=++index;
                                            if(index<devices.length) {
                                                checkForSameName(resolve, devices, index)
                                            }else{
                                                addDeviceNameConflictInfoComponents();
                                                resolve();
                                            }
                                        }
                                    }
                                    else{
                                        addDeviceNameConflictInfoComponents();
                                        resolve();
                                    }

                                },
                                limit: 1,
                                offset: 0
                            }
                        })
                } catch (e) {
                    console.log(e);
                }
            }


            /**
             * Prompt Dialog box Same Device Name
             * @param device
             * @returns {*}
             * @private
             */
            var _promptDialog = function (resolve, devices, index) {
                var comp = devices[index];
                var ipAddress = comp.get("ipAddress");
                var deviceName = comp.get("deviceName");
                var projectName = comp.get("projectName");
                var slotPath = comp.get("slotPath");

                var content = "<div class='soe-overwrite-dialog'><div>" +
                    lex.get("sbc.offlineEngineering.import.write") + "</div><div>" +
                    lex.get("sbc.offlineEngineering.import.write.Input") + "</div>" +
                    "<div> <input type='text' id='soe-suffix-devicename'>" +
                    "</div></div>";

                content = content.replace("$$deviceName$$", deviceName)
                    .replace("$$ipAddress$$", ipAddress);

                promptDialog = dialogs.make(
                    {
                        title: lex.get("sbc.offlineEngineering.import.write.name.Dialog"),
                        content: content,
                        buttons: [
                            {
                                name: "overwrite",
                                displayName: lex.get("sbc.offlineEngineering.dialog.save"),
                                handler: function (dlg) {
                                    dlg.close();
                                    var dlgContent = dlg.content();
                                    var suffix = $(dlgContent).find("#soe-suffix-devicename").val();
                                    if (suffix.trim() != "") {
                                        var deviceNewName = deviceName + "_" + suffix;
                                        setDeviceName(slotPath, deviceName, deviceNewName, ipAddress, projectName);
                                        var nextIndex = ++index;
                                        if (nextIndex < devices.length) {
                                            checkForSameName(resolve, devices, nextIndex);
                                        }else{
                                            addDeviceNameConflictInfoComponents(); // If done checking with all devices then add conflicts info to service slot.
                                            resolve();
                                        }

                                    } else {
                                        alert(lex.get("sbc.offlineEngineering.import.write.name.error"));
                                        _promptDialog(resolve, devices, index);
                                    }

                                }
                            }]
                    });
                promptDialog.show();

            }


            /**
             * Child component creation for device name conflict info.
             * @param slotPath
             * @param deviceOldName
             * @param deviceNewName
             * @param deviceIpAddress
             * @param projectName
             */
            function setDeviceName(slotPath,deviceOldName, deviceNewName, deviceIpAddress, projectName){
                var infoChildComp = new baja.Component();
                infoChildComp.add({slot: "slotPath", value: slotPath});
                infoChildComp.add({slot: "deviceOldName", value: deviceOldName});
                infoChildComp.add({slot: "deviceNewName", value: deviceNewName});
                infoChildComp.add({slot: "deviceIpAddress", value: deviceIpAddress});
                infoChildComp.add({slot: "projectName", value: projectName});
                devicesNewNamesMainComp.add({slot: "deviceConflictInfo?", value: infoChildComp});
            }

            /**
             * Add device names conflict component to service slot
             */
            function addDeviceNameConflictInfoComponents()
            {
                if (sbcService.get("devicesNewNames") != null) {
                    sbcService.remove("devicesNewNames");
                }
                sbcService.add({slot: "devicesNewNames", value: devicesNewNamesMainComp});
            }

            var _startImporting = function (type) {
                _updateBttnsOnImport();
                switch (type) {
                    case 'datapoints':
                        progress.prototype.startDatapointsTimer();
                        sbcService.invoke({
                            slot: "importDataPoints"
                        });
                        break;
                    case 'alarms':
                        progress.prototype.startAlarmsTimer();
                        sbcService.invoke({
                            slot: "importAlarms"
                        });
                        break;
                    case 'hdlogs':
                        progress.prototype.startHDLogsTimer();
                        sbcService.invoke({
                            slot: "importHDLogs"
                        });
                        break;
                    default:
                        console.log("SBC service doesn't support import of " + type);
                        break;
                }
            };

            var _stopImporting = function (type) {
                switch (type) {
                    case 'datapoints':
                        var datapointsContainer = sbcService.getDataPoints();
                        importList.datapoints = datapointsContainer.getCompletedDataPointsCount();
                        progress.prototype.stopDatapointsTimer(sbcService.getIsImportCancelled() || isImportProcessCancelled);
                        break;
                    case 'alarms':
                        var alarmsContainer = sbcService.getAlarms();
                        importList.alarms = alarmsContainer.getCompletedAlarmListCount();
                        progress.prototype.stopAlarmsTimer(sbcService.getIsImportCancelled() || isImportProcessCancelled);
                        break;
                    case 'hdlogs':
                        var hdlogsContainer = sbcService.getHdLogs();
                        importList.hdlogs = hdlogsContainer.getCompletedHDLogsCount();
                        progress.prototype.stopHDLogsTimer(sbcService.getIsImportCancelled() || isImportProcessCancelled);
                        break;

                    default:
                        console.log("SBC service doesn't support import of " + type);
                }

                if (!importStatus.datapoints && !importStatus.alarms && !importStatus.hdlogs && !isImportProcessCancelled){
                    if(cancelDialog != null) {
                        cancelDialog.close();
                        cancelDialog = null;
                    }
                    $widgetContainer.find("#sbcOfflineEngineering-nextBttn").click();
                }
            };


            var _startStopImporting = function (stopImport) {

                if (stopImport) {
                    if (importStatus.datapoints) {
                        importStatus.datapoints = 0;
                        _stopImporting('datapoints');
                    } else if (importStatus.alarms) {
                        importStatus.alarms = 0;
                        _stopImporting('alarms');
                    } else if (importStatus.hdlogs) {
                        importStatus.hdlogs = 0;
                        _stopImporting('hdlogs');
                    }
                }
                if (!isImportProcessCancelled) {
                    if (importStatus.datapoints) {
                        _startImporting('datapoints');
                    }
                    else if (importStatus.alarms) {
                        _startImporting('alarms');
                    }
                    else if (importStatus.hdlogs) {
                        _startImporting('hdlogs');
                    }
                } else {
                    $widgetContainer.find("#importView #sbcOfflineEngineering-status-message").html(lex.get("sbc.offlineEngineering.cancelled.status"));
                }
            };

            var alarmListStatus = function(selected,completed){
                    progress.prototype.setAlarmsCount(completed + " of " + selected);
                    var percentage = parseInt(parseInt(completed) / parseInt(selected) * 100);
                    if (isNaN(percentage)) {
                        percentage = 0;
                    }
                    $widgetContainer.find("#pcAlarms").prop("innerHTML", (percentage + "%"));
             }

             var dataPointsStatus = function(selected,completed){
                     progress.prototype.setDatapointsCount(completed + " of " + selected);
                     var percentage = parseInt(parseInt(completed) / parseInt(selected) * 100);
                     if (isNaN(percentage)) {
                         percentage = 0;
                     }
                     $widgetContainer.find("#pcDataPoints").prop("innerHTML", (percentage + "%"));
              }

              var hdLogsStatus = function(selected,completed){
                      progress.prototype.setHDLogsCount(completed + " of " + selected);
                      var percentage = parseInt(parseInt(completed) / parseInt(selected) * 100);
                      if (isNaN(percentage)) {
                          percentage = 0;
                      }
                      $widgetContainer.find("#pcHDLogs").prop("innerHTML", (percentage + "%"));
               }

            var _attachSubscriber = function (prop, cx) {
                var container = this;
                var propName = prop.getName();
                console.log("Property name: " + prop.getName());

                if (propName === "dataPointsImportState") {
                    if (!container.getDataPointsImportState()) {
                        //stop datapoints timer
                        dataPointsStatus(container.getSelectedDataPointsCount(),container.getCompletedDataPointsCount());
                        _startStopImporting(true);
                    }
                } else if (propName === "completedDataPointsCount") {
                    //update datapoints count text and progress percentage
                    dataPointsStatus(container.getSelectedDataPointsCount(),container.getCompletedDataPointsCount());
                } else if (propName === "alarmListImportState") {
                    if (!container.getAlarmListImportState()) {
                        //stop alarms timer
                        alarmListStatus(container.getSelectedAlarmListCount(),container.getCompletedAlarmListCount());
                        _startStopImporting(true);
                    }
                } else if(propName === "completedAlarmListCount") {
                    //update alarms count text and progress percentage
                    alarmListStatus(container.getSelectedAlarmListCount(),container.getCompletedAlarmListCount());
                } else if (propName === "hdLogsImportState") {
                    if (!container.getHdLogsImportState()) {
                        //stop hdlogs timer
                        hdLogsStatus(container.getSelectedHDLogsCount(),container.getCompletedHDLogsCount());
                        _startStopImporting(true);
                    }
                } else if (propName === "completedHDLogsCount") {
                    //update hdlogs count text and progress percentage
                    hdLogsStatus(container.getSelectedHDLogsCount(),container.getCompletedHDLogsCount());
                }
            };

            var _resetData = function () {
                selectedModules = {};
                importStatus = {};
                importList = {datapoints: 0, alarms: 0, hdlogs: 0};
            };

            var _cancelImportOperation = function(sbc_service){
                $widgetContainer.find("#importView #sbcOfflineEngineering-status-message").html(lex.get("sbc.offlineEngineering.cancel.status"));
                _updateBttnsOnCancel();
                sbc_service.invoke({
                    slot: "clearScannedInfo",
                    value: 2 //Cancel Import
                });
            };

            return {

                /**
                 *
                 * @param element: dom element the view will be added to
                 * @param params: javascript object with widget parameters required to load the view within the widget
                 */

                loadView: function (element, params) {
                    _resetData();
                    $element = $(element);
                    $widgetContainer = (params.widgetContainer) ? $(params.widgetContainer) : (element).closest(".sbcOfflineEngineering-wizard");
                    selectedModules = params.selectedModules;
                    importStatus = params.selectedModules;

                    sbcService = params.component;
                    var subscriber = params.subscriber;
                    isImportProcessCancelled = false;
                    sbcService.setIsImportCancelled(false); //reset the flag
                    $(element).append(template({
                        defaultActionMsg: lex.get("sbc.offlineEngineering.import.defaultMsg"),
                        datapoints: lex.get("sbc.offlineEngineering.select.datapoints"),
                        hdlogs: lex.get("sbc.offlineEngineering.select.hdlogs"),
                        alarms: lex.get("sbc.offlineEngineering.select.alarms"),
                        selectedAlarms: true,
                        selectedHdlogs: true
                    }));
                    $widgetContainer.find("#pcDataPoints").prop("innerHTML", "0%");
                    $widgetContainer.find("#pcAlarms").prop("innerHTML", "0%");
                    $widgetContainer.find("#pcHDLogs").prop("innerHTML", "0%");
                    $widgetContainer.find("#sbcOfflineEngineering-nextBttn").css("display","none");
                    subscriber.attach('changed', _attachSubscriber);

                    progress.prototype.buildProgress("#import-datapoints-circle", "#import-alarms-circle",
                        "#import-hdlogs-circle", "#sbcOfflineEngineering-importView");


                    var datapointsContainer = sbcService.getDataPoints();
                    var alarmsContainer = sbcService.getAlarms();
                    var hdlogsContainer = sbcService.getHdLogs();

                    //TODO - add alarms and HDLogs
                    var importInProgress = datapointsContainer.getDataPointsImportState()
                        || alarmsContainer.getAlarmListImportState()
                        || hdlogsContainer.getHdLogsImportState();


                    if (!importInProgress) {
                        var initialCount = 0;
                        sbcService.invoke({
                            slot: "scanExistDevices"
                        }).then(function (returnValue) {
                            if (returnValue) {
                                _getDeviceData().then(function () {
                                    subscriber.subscribe(datapointsContainer);
                                    subscriber.subscribe(alarmsContainer);
                                    subscriber.subscribe(hdlogsContainer);
                                    progress.prototype.setDatapointsCount(initialCount + " of " + initialCount);
                                    progress.prototype.setAlarmsCount(initialCount + " of " + initialCount);
                                    progress.prototype.setHDLogsCount(initialCount + " of " + initialCount);
                                    _startStopImporting();
                                })
                            } else {
                                // first import, so "isFirstImport" is true
                                subscriber.subscribe(datapointsContainer);
                                subscriber.subscribe(alarmsContainer);
                                subscriber.subscribe(hdlogsContainer);
                                progress.prototype.setDatapointsCount(initialCount + " of " + initialCount);
                                progress.prototype.setAlarmsCount(initialCount + " of " + initialCount);
                                progress.prototype.setHDLogsCount(initialCount + " of " + initialCount);
                                _startStopImporting();
                            }
                        });
                    }
                },

                /**
                 * validate user input
                 *
                 * @returns {boolean}: return true/false based on validity of user input
                 */

                validate: function () {
                    return new Promise(function (resolve) {
                        resolve(true);
                    });
                },

                /**
                 * clean up and save before going to next view
                 *
                 * @returns {Promise}: return promise to be resolved before going to next view
                 */
                next: function (event, params) {
                    return new Promise(function (resolve) {
                        params.subscriber.detach("changed", _attachSubscriber);
                        resolve();
                    });
                },

                /**
                 * clean up and save before going to previous view
                 *
                 */
                back: function (event, params) {
                    return new Promise(function (resolve) {
                        params.subscriber.detach("changed", _attachSubscriber);
                        resolve();
                    });
                },

                /**
                 * cancel operation
                 *
                 */

                cancel: function (sbc_service) {
                    sbc_service.setIsImportCancelled(false); //reset the flag
                    cancelDialog = Dialogs.showYesNo({
                        title : lex.get("sbc.offlineEngineering.dialog.confirm"),
                        content:lex.get("sbc.offlineEngineering.import.cancel.msg")
                    })
                        .yes(function (dialog) {
                            isImportProcessCancelled = true;
                            dialog.close();
                            cancelDialog = null;
                            _cancelImportOperation(sbc_service);
                            _startStopImporting(true);
                        }).no(function(dialog) {
                            isImportProcessCancelled = false;
                            dialog.close();
                            cancelDialog = null;
                    });
                },

                /**
                 * data to be passed to widget to communicate between views
                 *
                 * @returns {json}: return key value pairs of data
                 */

                getData: function () {
                    //return {scannedModules: importList};
                    return {};
                },


                params: {
                    vname: 'importView',
                    id: 'importView',
                    message: lex.get('sbc.offlineEngineering.import'),
                    backBttnMsg: lex.get('sbc.offlineEngineering.back'),
                    nextBttnMsg: lex.get('sbc.offlineEngineering.next'),
                    isCancelAvailable: true,
                    cancelBttnMsg: lex.get('sbc.offlineEngineering.cancel')
                }
            };
        };

        return importView;
    });
