/**
 * * Copyright (c) 2018. Honeywell International, Inc. All Rights Reserved.
 */


define(['baja!',
        'jquery',
        'Promise',
        'underscore',
        'dialogs',
        'nmodule/webEditors/rc/fe/fe',
        'nmodule/webEditors/rc/fe/feDialogs',
        'bajaux/util/CommandButton',
        'bajaux/commands/Command',
        'hbs!nmodule/sbcOfflineEngineering/rc/hbs/selectView',
        'hbs!nmodule/sbcOfflineEngineering/rc/hbs/hyperLink',
        'lex!sbcOfflineEngineering'
        ],
    function (baja,
              $,
              Promise,
              _,
              dialogs,
              fe,
              feDialogs,
              CommandButton,
              Command,
              template,
              template1,
              Lexicons) {

        'use strict';

        /**
         * Description of your widget.
         *
         * @class
         * @alias module:nmodule/sbcOfflineEngineering/rc/js/OfflineEngineeringSelectView
         */
        var selectView = new function(){

            var lex = Lexicons[0];
            var $element = null;
            var $widgetContainer =  null, sbcService = null;
            var _verifyingCodeDialog = null, _codePageDialog = null;
            var _codePageParams = {
                options :"",
                resolve :""
            };

            var _getOrdEditor = function(jq) {
                return jq.find('.chooser').data('widget');
            };

            var _attachCheckboxHandlers = function() {
                $widgetContainer.find("#selectView #soe-select-enableall").change( function (event) {
                    $widgetContainer.find("#selectView").find(".soe-option").prop("checked", this.checked);
              });

                $widgetContainer.find("#selectView .soe-option").click(function (event) {
                  var checkedAll = true;
                    $widgetContainer.find('#selectView .soe-option').each(function (index, e) {
                     if(!this.checked) {
                        checkedAll = false;
                         return false;
                     }
                  });
                    $widgetContainer.find("#selectView #soe-select-enableall").prop("checked", checkedAll);
              });
            };

            var _getWords = function (identifiedWords) {
                var listOfWords = "";
                if(identifiedWords !== "") {
                    var words = identifiedWords.split(";");
                    for(var i=0; i < words.length; i++) {
                        if(i < words.length) {
                            listOfWords = listOfWords + words[i] + "\n";
                        }
                        listOfWords = listOfWords + words[i];
                    }
                } else {
                    listOfWords = listOfWords + lex.get("sbc.offlineEngineering.codePage.nonEngChar");
                }
                return listOfWords;
            };

            var _getPopupContent = function() {
                return "<div>" +
                    "<label class='codePageLabel'>"+lex.get("sbc.offlineEngineering.codePage.comboxLabel")+"</label>"+
                    "<div id='codePage' class='codePage'>" +
                    "<table width='100%'"+
                    "<tr><td width='60%'>"+
                    "<select id='code' class='code'>" +
                    _codePageParams.options+
                    "</select>"+
                    "</td><td width='40%'>"+
                    "<input id='revert' type='button' class='ux-btn' value='"+lex.get("sbc.offlineEngineering.codePage.button.revert")+"'>"+
                    "</td></tr></table>"+
                    "</div>"+
                    "<div>" +
                    "<div>" +
                    "<label id='pointLabel' class='codePageLabel spinner'>" +
                        lex.get("sbc.offlineEngineering.codePage.sampleTextLabel")+
                    "</label>" +
                    "</div>"+
                    "<div><textarea id='pointText' class='pointText' rows='4' cols='40' readonly>" +
                    "</textarea></div>"+
                    "</div>"+
                    "</div>";
            };
            
            var _getCodePageDialog = function (content) {
                return dialogs.make({
                    title: lex.get("sbc.offlineEngineering.codePage.title"),
                    content: content,
                    buttons: [
                        {
                            name:"proceed",
                            displayName:lex.get("sbc.offlineEngineering.codePage.button.proceed"),
                            handler:function(dialog) {
                                dialog.close();
                                _codePageParams.resolve();
                            }
                        },
                        {
                            name:"cancel",
                            displayName:lex.get("sbc.offlineEngineering.codePage.button.cancel"),
                            handler:function(dialog) {
                                dialog.close();
                            }
                        }
                    ]
                });
            };

            var _addCodePageEvents = function() {
                $('#code').change(function (event) {
                    $(".js-dialog-button-proceed").prop( "disabled", true);
                    $("#revert").prop("disabled", true);
                    $('#pointLabel').addClass("spinner");
                    var component = new baja.Component();
                    component.add({slot:'type', value:"2"});
                    component.add({slot:'codepage', value:$("#code :selected").val()});
                    $("#code").prop( "disabled", true);
                    sbcService.invoke({
                        slot:"scanCodePage",
                        value:component
                    });
                });

                $("#revert").click(function (event) {
                    $('#code option[value="'+sbcService.getCodePage().getIdentifiedCodePage()+'"]').prop('selected', true);
                    $("#pointText").val(_getWords(sbcService.getCodePage().getSuggestedWords()));
                });
            };

            var _attachSubscriber = function (prop, cx) {
                var service = this;
                var propName = prop.getName();
                if (propName === "codePageScanningState"){
                    if(!service.getCodePageScanningState()){
                        if(_verifyingCodeDialog && !_verifyingCodeDialog.isClosed()) {
                            _verifyingCodeDialog.close();
                            _verifyingCodeDialog = null;

                            var popupContent = _getPopupContent();
                            var codePageDialog = _getCodePageDialog(popupContent);
                            codePageDialog.show({content:popupContent});
                            $('.js-dialog').css("background-color", '#e9eaea');
                            $('.js-dialog-button-content').css("background-color", '#e9eaea');
                            _addCodePageEvents();
                        }

                        if(service.getIsCodePageAvailable()){
                           $('#code option[value="'+service.getSelectedCodePage()+'"]').prop('selected', true);
                           $("#code").prop( "disabled", true);
                           $("#revert").prop("disabled", true);
                           $(".js-dialog-button-proceed").prop( "disabled", false);
                           var listOfWords = lex.get("sbc.offlineEngineering.codePage.project");
                           $("#pointText").val(listOfWords);
                           $('#pointLabel').removeClass("spinner");
                        }else{
                           $('#code option[value="'+service.getSelectedCodePage()+'"]').prop('selected', true);
                           $("#pointText").val(_getWords(service.getProcessedWords()));
                           $('#pointLabel').removeClass("spinner");
                           $("#code").prop( "disabled", false);
                           $("#revert").prop("disabled", false);
                           $(".js-dialog-button-proceed").prop( "disabled", false);
                        }
                    }
                }
            };


            return {

                /**
                 *
                 * @param element: dom element the view will be added to
                 * @param params: javascript object with widget parameters required to load the view within the widget
                 */

                loadView : function(element, params) {
                    var component = params.component;
                    sbcService = component;
                    $element = $(element);
                    $widgetContainer = (params.widgetContainer) ?  $(params.widgetContainer) : (element).closest(".sbcOfflineEngineering-wizard");
                    var subscriber = params.subscriber;

                    $(element).append(template({
                        defaultActionMsg:lex.get("sbc.offlineEngineering.select.defaultMsg"),
                        enableall:lex.get("sbc.offlineEngineering.select.enableall"),
                        datapoints: lex.get("sbc.offlineEngineering.select.datapoints"),
                        hdlogs: lex.get("sbc.offlineEngineering.select.hdlogs"),
                        alarms:lex.get("sbc.offlineEngineering.select.alarms"),
                    }));

                    //attach checkbox handlers
                    _attachCheckboxHandlers();

                    subscriber.attach('changed', _attachSubscriber);

                    var codePageContainer = sbcService.getCodePage();
                    if(!codePageContainer.getCodePageScanningState()) {
                        subscriber.subscribe(codePageContainer);
                    }

                    fe.buildFor({
                        value: component.getInputLocation(),
                        dom: $(element).find('.chooser'),
                        formFactor: 'mini'
                    }).then(function() {
                        var location = component.getInputLocation();
                        if(location.$ord === "null") {
                            $widgetContainer.find("#sbcOfflineEngineering-nextBttn").prop('disabled', true);
                        }
                    });
                    $(element).find(".chooser").attr('title',component.getInputLocation());
                    var cmd = new Command({
                        module: 'sbcOfflineEngineering',
                        lex: 'sbc.offlineEngineering.select.popOut',
                        func: function () {
                            return _getOrdEditor($(element)).read()
                                .then(function (currentOrd) {
                                    return feDialogs.showFor({
                                        value: currentOrd,
                                        formFactor: 'compact',
                                        readonly:true
                                    });
                                })
                                .then(function (newOrd) {
                                    if (newOrd) {
                                        component.setInputLocation(newOrd);
                                        $(element).find(".chooser").attr('title',newOrd);
                                        if(newOrd.$ord != "null") {
                                            $widgetContainer.find("#sbcOfflineEngineering-nextBttn").prop('disabled', false);
                                        } else {
                                            $widgetContainer.find("#sbcOfflineEngineering-nextBttn").prop('disabled', true);
                                        }
                                        return _getOrdEditor($(element)).load(newOrd);
                                    } else {
                                        /** On cancel old value be set which is not visible here so reading from component **/
                                        var location = component.getInputLocation();
                                        if(location.$ord === "null") {
                                            $widgetContainer.find("#sbcOfflineEngineering-nextBttn").prop('disabled', true);
                                        }
                                    }
                                });
                        }
                    });

                    fe.buildFor({
                        value: cmd,
                        dom: $(element).find('.chooserButton'),
                        type: CommandButton
                    }).then(function() {
                        //patch to replace default niagara edit icon/display with custom button
                        $(element).find(".chooserButton .display .displayName").addClass("ux-btn");
                    });

                    $("#helpHyperLink").on("click", function () {
                        dialogs.showOk({
                            title: "Help",
                            content: template1({
                                title:"Saia PCD Supervisor Import Wizard 2.1",
                                header:"Important information about import wizard process in SAIA PCD Supervisor 2.1",
                                message1:"Prior to version 2.1, projects were imported in SBC supervisor by using CPU device name as slot in the ORD path from the PG5 project.",
                                message2:"This could cause an issue when importing device with the same CPU device name from different PG5 project into the same SBC supervisor station where unique name are required.",
                                message3:"With the current newer version SAIA PCD Supervisor 2.1 the PG5 import wizard process apply a rule to compare the CPU device name as configured in PG5 device configuration.",
                                message4:"During the import process the device will be now mapped within the SbcIpNetwork applying a unique name conversion through SbcHeadstation or SbcSubstation appending to that a suffix.",
                                message5:"In case customers require to still use the old naming convention, we recommend performing that via Rename feature.",
                                message6:"The system as it is designed now will resolve the unique address by seeing, as explained above, the reference of device name, independently from the name.",
                                message7:"For further detailed information please refer to the SAIA PCD Supervisor manual or contact SAIA technical support to get additional support.",
                                message8:"During the import stage the import wizard will compare the imported information in relation to the name of the device.",
                                message9:"In case the device does already exist in the Saia PCD Supervisor database a popup will be offered to enter a suffix to create a new device using the suffix.",
                                message10:"This is not covered for Sub Stations and only available for Head Stations."
                            })
                        });
                    });

                },

                /**
                 * validate user input
                 *
                 * @returns {boolean}: return true/false based on validity of user input
                 */

                validate: function() {
                    //TODO: validate user input

                    var fileLocation = $('.ord').val();
                    return new Promise(function (resolve) {
                        var fileExtension = ".saia5pj";
                        if( $element.find('#selectView .soe-option:checked').length < 1) {
                            dialogs.showOk({
                                title : lex.get("sbc.offlineEngineering.dialog.error"),
                                content:lex.get("sbc.offlineEngineering.select.missing.input"),
                                ok:lex.get("sbc.offlineEngineering.dialog.ok")
                            });
                            resolve(false);
                        } else {
                            if(fileLocation && (fileLocation.lastIndexOf('file:^') == -1 || fileLocation.lastIndexOf(fileExtension) == -1
                                || ((fileLocation.lastIndexOf(fileExtension) + fileExtension.length) <  fileLocation.length))){ //string.endsWith doesn't work in Niagara!

                                dialogs.showOk({
                                    title : lex.get("sbc.offlineEngineering.dialog.error"),
                                    content:lex.get("sbc.offlineEngineering.select.wrong.input"),
                                    ok:lex.get("sbc.offlineEngineering.dialog.ok")
                                });
                                resolve(false);
                            } else {
                                baja.Ord.make(fileLocation)
                                    .get()
                                    .then(function (result) {
                                        if(result.isDirectory()){
                                            dialogs.showOk({
                                                title : lex.get("sbc.offlineEngineering.dialog.error"),
                                                content:lex.get("sbc.offlineEngineering.select.wrong.input"),
                                                ok:lex.get("sbc.offlineEngineering.dialog.ok")
                                            });
                                            resolve(false);
                                        } else {
                                            resolve(true);
                                        }
                                    }).error(function(){
                                    dialogs.showOk({
                                        title : lex.get("sbc.offlineEngineering.dialog.error"),
                                        content:lex.get("sbc.offlineEngineering.select.wrong.input"),
                                        ok:lex.get("sbc.offlineEngineering.dialog.ok")
                                    });
                                    resolve(false);
                                });
                            }
                        }
                    });


                    /*
                     if( $element.find('#selectView .soe-option:checked').length < 1) {
                     dialogs.showOk({
                     title : lex.get("sbc.offlineEngineering.dialog.error"),
                     content:lex.get("sbc.offlineEngineering.select.missing.input"),
                     ok:lex.get("sbc.offlineEngineering.dialog.ok")
                     });
                     return false;
                     } else if(fileLocation && fileLocation.lastIndexOf('.') > -1) {
                     var file_extension = fileLocation.substr(fileLocation.lastIndexOf('.') + 1);
                     if(file_extension && !(file_extension === 'saia5pj' )) {
                     dialogs.showOk({
                     title : lex.get("sbc.offlineEngineering.dialog.error"),
                     content:lex.get("sbc.offlineEngineering.select.wrong.input"),
                     ok:lex.get("sbc.offlineEngineering.dialog.ok")
                     });
                     return false;
                     }
                     }else if(fileLocation && fileLocation.lastIndexOf('file:^') == -1) {
                     dialogs.showOk({
                     title : lex.get("sbc.offlineEngineering.dialog.error"),
                     content:lex.get("sbc.offlineEngineering.select.wrong.input"),
                     ok:lex.get("sbc.offlineEngineering.dialog.ok")
                     });
                     return false;
                     }

                     return true;
                     */
                },

                /**
                 * clean up and save before going to next view
                 *
                 * @returns {Promise}: return promise to be resolved before going to next view
                 */
                next : function(event, params) {
                 return new Promise(function(resolve){
                 //TODO:task to do before moving to next screen
                    /* if($widgetContainer.find("#selectView #soe-select-datapoints").prop("checked") ||
                         $widgetContainer.find("#selectView #soe-select-hdlogs").prop("checked")) {*/
                         _verifyingCodeDialog = dialogs.show({
                             content: "<label class='spinner'>"+lex.get('sbc.offlineEngineering.codePage.codeScan')+"</label>"
                         });

                         sbcService.invoke({
                                          slot:"createEncodings"
                                      });
                         baja.Ord.make("file:^resources/encodings.xml").get({
                             ok: function(response) {
                                 $.get(response.getReadUri(), function(encodings) {
                                     var options ="";
                                     $.each($(encodings).find("Encoding"), function(p, e) {
                                         options = options + "<option value='"+$(e).attr("code")+"'>"+$(e).attr("description")+"</option>";
                                     });
                                     _codePageParams.options = options;
                                     _codePageParams.resolve = resolve;
                                     var component = new baja.Component();
                                     component.add({slot:'type', value:"1"});
                                     component.add({slot:'codepage', value:"UTF8"});
                                     $("#code").prop( "disabled", true);
                                     sbcService.invoke({
                                         slot:"scanCodePage",
                                         value:component
                                     });
                                 });
                             }
                         });
                    /* } else {
                         resolve();
                     }*/
                 });
                 },

                /**
                 * clean up and save before going to previous view
                 *
                 */
                back : function(event, params) {
                    return new Promise(function(resolve){
                        //TODO:task to do before moving to previous screen
                        resolve();
                    });
                },

                /**
                 * cancel operation
                 *
                 */

                cancel: function() {
                    //TODO: cancel running operation within current view
                },

                /**
                 * data to be passed to widget to communicate between views
                 *
                 * @returns {json}: return key value pairs of data
                 */

                getData: function() {
                    //TODO: task to update data to be passed to widget
                    return { selectedModules : {
                        datapoints:$widgetContainer.find("#selectView #soe-select-datapoints").prop("checked"),
                        alarms:$widgetContainer.find("#selectView #soe-select-alarms").prop("checked"),
                        hdlogs:$widgetContainer.find("#selectView #soe-select-hdlogs").prop("checked")
                    }
                    };
                },


                /**
                 * enable/disable status of visisble action buttons within the view
                 *
                 * @returns {json}: return action buttons status (enabled/disabled)
                 */
                /*
                //Note: by default all buttons within the view are enabled.
                // uncomment and update the code to conditionally enable or disable action buttons
                enableDisableActionBttns: function () {
                //TODO: update action buttons status
                    return {
                        disabledBack : false,
                        disabledNext : false,
                        disabledCancel:false
                    };
                },
*/
                params:{
                    vname:'selectView',
                    id:'selectView',
                    message:lex.get('sbc.offlineEngineering.select'),
                    nextBttnMsg: lex.get('sbc.offlineEngineering.select.next')
/*                    cancelBttnMsg: lex.get('sbc.offlineEngineering.cancel')*/
                }
            };
        };

        return selectView;
    });
