/**
 * * Copyright (c) 2018. Honeywell International, Inc. All Rights Reserved.
 */
define(['nmodule/js/rc/d3/d3.min',
        'jquery',
        'lex!sbcOfflineEngineering',
        'css!nmodule/sbcOfflineEngineering/rc/css/sbcOfflineEngineering'
        ], function(
        d3,
        $,
        lexicons
    ) {
    'use strict';
        var lex = lexicons[0];

        var Progress = function() {
        };

        var startAngle = 0, endAngle = 0;
        var circleStrokeWidth = 10;
        var pathStrokeWidth = 10;
        var radius = 100,svgHeight=240;
        var x = 120, y = 120, width = 0, height = 0;
        var timer1, timer2, timer3;
        var datapointsSVG, alarmsSVG, hdlogsSVG, datapointsCircle, alarmsCircle, hdlogsCircle;
        var datapointsPath, alarmsPath, hdlogsPath, datapointsText, alarmsText, hdlogsText;
        var alarmsImage, datapointsImage, hdlogsImage;
        var imgheight='calc(100% - 48)', imgwidth='calc(100% - 48)', imgx = 24, imgy = 24;


    var _validateImageDimensions = function() {

        var bbox, dpimage;
        if(datapointsImage && datapointsImage[0]) {
            dpimage = datapointsImage[0];
            if($.isArray(dpimage) && dpimage.length > 0) {
                dpimage = dpimage[0];
            }
            if (dpimage.getBoundingClientRect) {
                bbox = dpimage.getBoundingClientRect();
                // console.log(bbox);
                if (bbox.width <= 0) {
                    datapointsImage.attr("width", "80%").attr("height", "80%");
                    alarmsImage.attr("width", "80%").attr("height", "80%");
                    hdlogsImage.attr("width", "80%").attr("height", "80%");
                }
            }
        }
    };

        var _calculatePosition = function(x, y, radius, startAng, endAng){
            // console.log("calculatePosition:  timer1 ============ " + timer1);
//            var angleInRadians = (endAng - 90) * Math.PI / 180.0;
//            var startAngleInRadians = (startAng - 90) * Math.PI / 180.0;
            var start = _calculateCoordinates(x, y, radius, endAng);
            var end = _calculateCoordinates(x, y, radius, startAng);
            var arcSweep = endAng - startAng <= 180 ? "0" : "1";
            var d = ["M", start.x, start.y,"A", radius, radius, 0, arcSweep, 0, end.x, end.y].join(" ");
            return d;
        };

        function _calculateCoordinates(centerX, centerY, radius, angleInDegrees){
            var angleInRadians = (angleInDegrees - 90) * Math.PI / 180.0;
            return {
                x: centerX + (radius * Math.cos(angleInRadians)),
                y: centerY + (radius * Math.sin(angleInRadians))
            };
        }

        //datapoints timer.
        Progress.prototype.startDatapointsTimer = function(){
            datapointsCircle.attr("stroke","#F2F2F2");
            datapointsCircle.attr("stroke-width",circleStrokeWidth);
            timer1 = setInterval(
                function(){
                    startAngle = (startAngle + 10) % 360;
                    endAngle = (90 + startAngle) % 360;
                    var d = _calculatePosition(x, y, radius, startAngle, endAngle);
                    datapointsPath.attr('d', d);
                },100
            );
        };

        Progress.prototype.stopDatapointsTimer = function(cancelFlag){
          if(cancelFlag){
            datapointsCircle.attr("stroke", "#B3B3B3");
            datapointsPath.attr("stroke","#B3B3B3");
    	  }else{
    	    datapointsCircle.attr("stroke", "#0066B3");
            datapointsImage.attr("xlink:href", "/module/sbcOfflineEngineering/rc/images/scan_datapoints_done.svg");
            datapointsText.attr("fill","#ffffff");
          }
    	  datapointsCircle.attr("stroke-width", pathStrokeWidth);
    	  window.clearInterval(timer1);
        };

        //Alarms timer
        Progress.prototype.startAlarmsTimer = function(){
            alarmsCircle.attr("stroke","#F2F2F2");
            alarmsCircle.attr("stroke-width",circleStrokeWidth);
            timer2 = setInterval(
                    function(){
                        startAngle = (startAngle + 10) % 360;
                        endAngle = (90 + startAngle) % 360;
                        var d = _calculatePosition(x, y, radius, startAngle, endAngle);
                        alarmsPath.attr('d', d);
                    },100
             );
    	};

    	Progress.prototype.stopAlarmsTimer = function(cancelFlag){
    	      if(cancelFlag){
    	        alarmsCircle.attr("stroke", "#B3B3B3");
    	        alarmsPath.attr("stroke","#B3B3B3");
    	      }else{
    		    alarmsCircle.attr("stroke", "#0066B3");
                alarmsImage.attr("xlink:href", "/module/sbcOfflineEngineering/rc/images/scan_alarms_done.svg");
                alarmsText.attr("fill","#ffffff");
    		  }
    		  alarmsCircle.attr("stroke-width", pathStrokeWidth);
    		  window.clearInterval(timer2);
        };

    //Alarms timer
        Progress.prototype.startHDLogsTimer = function(){
            hdlogsCircle.attr("stroke","#F2F2F2");
            hdlogsCircle.attr("stroke-width",circleStrokeWidth);
            timer3 = setInterval(
                function(){
                    startAngle = (startAngle + 10) % 360;
                    endAngle = (90 + startAngle) % 360;
                    var d = _calculatePosition(x, y, radius, startAngle, endAngle);
                    hdlogsPath.attr('d', d);
                },100
            );
        };

        Progress.prototype.stopHDLogsTimer = function(cancelFlag){
            if(cancelFlag){
                hdlogsCircle.attr("stroke", "#B3B3B3");
                hdlogsPath.attr("stroke","#B3B3B3");
            }else{
                hdlogsCircle.attr("stroke", "#0066B3");
                hdlogsImage.attr("xlink:href", "/module/sbcOfflineEngineering/rc/images/scan_hd_logs_done.svg");
                hdlogsText.attr("fill","#ffffff");
            }
            hdlogsCircle.attr("stroke-width", pathStrokeWidth);
            window.clearInterval(timer3);
        };



    Progress.prototype.stopTimersWhenItsCancelled = function(operations){
            //Stopping data points timer and setting dark shade color.
            window.clearInterval(timer1);
            window.clearInterval(timer2);
            window.clearInterval(timer3);

        if(operations.datapoints) {
                datapointsPath.attr("stroke","#B3B3B3");
                datapointsCircle.attr("stroke", "#B3B3B3");
                datapointsCircle.attr("stroke-width", pathStrokeWidth);
            }

            //Stopping alarms timer and setting dark shade color
            if(operations.alarms) {
                alarmsPath.attr("stroke","#B3B3B3");
                alarmsCircle.attr("stroke", "#B3B3B3");
                alarmsCircle.attr("stroke-width", pathStrokeWidth);
            }
            if(operations.hdlogs) {
            hdlogsPath.attr("stroke","#B3B3B3");
            hdlogsCircle.attr("stroke", "#B3B3B3");
            hdlogsCircle.attr("stroke-width", pathStrokeWidth);
            }

        };

        Progress.prototype.cancelScanTimer = function(operations){
          Progress.prototype.stopTimersWhenItsCancelled(operations);
          Progress.prototype.setDatapointsCount(0);
          Progress.prototype.setAlarmsCount(0);
          Progress.prototype.setHDLogsCount(0);
        };

        Progress.prototype.cancelImportTimer = function(operations){
          Progress.prototype.stopTimersWhenItsCancelled(operations);
        };

        var _resetParameters = function() {
            startAngle = 0;
            endAngle = 0;
        };

/*
        var _adjustProgress = function(event){
                //think twice before you modify this code
                _resetParameters();
                var dCircle = event.data.div;
                var parentDiv = event.data.parentD;
                var datapointsCell = $(dCircle).width();
                var parentHeight = $(parentDiv).height();
                svgHeight = parentHeight-50;
                x = datapointsCell/2;
                if(svgHeight<240){
                    svgHeight = 240;
                    y = svgHeight/2;
                    radius = y-20;
                }else{
                    y = svgHeight/2;
                    radius = y-20;
                }


                 if(radius>120){
                    imgheight = (radius-60)*2;
                    imgwidth = (radius-60)*2;
                 }
                imgx = (x-(imgwidth/2));
                imgy = (y-(imgheight/2));
                datapointsSVG.attr("height", svgHeight);
                alarmsSVG.attr("height", svgHeight);

                datapointsCircle.attr('cx',x).attr('cy',y);
                datapointsText.attr("x",(x-((datapointsText.text().toString().length*5)-5))).attr("y",(y+(radius-25)));
                var d = _calculatePosition(x,y,radius,startAngle,endAngle);
                datapointsPath.attr('d', d);

                alarmsCircle.attr('cx', x).attr('cy',y);
                alarmsText.attr("x",(x-((alarmsText.text().toString().length*5)-5))).attr("y",(y+(radius-25)));
                d = _calculatePosition(x,y,radius,startAngle,endAngle);
                alarmsPath.attr('d', d);

               // var datapointsCell = $(dCircle).width();
                datapointsCircle.attr('r',radius);
                alarmsCircle.attr('r',radius);
                /!*
                datapointsImage.attr("x", x-50);
                datapointsImage.attr("y", y-120);

                alarmsImage.attr("x", x-50);
                alarmsImage.attr("y", y-120)
                *!/
                datapointsImage.attr("x", imgx);
                datapointsImage.attr("y", imgy);
                datapointsImage.attr("height", imgheight);
                datapointsImage.attr("width", imgwidth);

                alarmsImage.attr("x", imgx);
                alarmsImage.attr("y", imgy);
                alarmsImage.attr("height", imgheight);
                alarmsImage.attr("width", imgwidth);
        };
*/

        Progress.prototype.buildProgress = function(dCircle,aCircle,hCircle, parentDiv){
                //think twice before you modify this code
                 _resetParameters();
//                 $( window ).resize({div: dCircle,parentD: parentDiv}, _adjustProgress);
                 // var datapointsCell = $(dCircle).width();
                 var parentHeight = $(parentDiv).height();
                 svgHeight = parentHeight-50;

                 datapointsSVG = d3.select(dCircle).append("svg").attr("viewBox", "0 0 240 240")/*.attr("width", "100%").attr("height", svgHeight)*/;
                 alarmsSVG = d3.select(aCircle).append("svg").attr("viewBox", "0 0 240 240")/*.attr("width", "100%").attr("height", svgHeight)*/;
                 hdlogsSVG = d3.select(hCircle).append("svg").attr("viewBox", "0 0 240 240")/*. attr("width", "100%").attr("height", svgHeight)*/;

                 datapointsCircle = datapointsSVG.append('circle').attr('cx', x)
                    .attr('cy',y)
                    .attr('r',radius)
                    .attr('fill','none')
                    .attr('stroke','#CDCFD1')
                    .attr('stroke-width',circleStrokeWidth);
                 datapointsPath = datapointsSVG.append("path")
                    .attr("stroke","#0066B3")
                    .attr("stroke-width",pathStrokeWidth)
                    .attr("fill","none");

                 datapointsImage = d3.select(dCircle)
                    .selectAll('svg')
                    .append("image")
                     .attr("id", "datapoints_image_default")
                    .attr("xlink:href", "/module/sbcOfflineEngineering/rc/images/scan_datapoints_default.svg")
                    .attr("x", imgx)
                    .attr("y", imgy)
                    .attr("height", imgheight)
                    .attr("width", imgwidth);

                 datapointsText = d3.select(dCircle)
                    .selectAll('svg')
                    .append("text")
                    .attr("x",(x-5))
                    .attr("y",(y+(radius-25)))
                    .text("0")
                    .attr("fill","#222222");

                 alarmsCircle = alarmsSVG.append('circle').attr('cx', x)
                    .attr('cy',y)
                    .attr('r',radius)
                    .attr('fill','none')
                    .attr('stroke','#CDCFD1')
                    .attr('stroke-width',circleStrokeWidth);

                 alarmsPath = alarmsSVG.append("path")
                    .attr("stroke","#0066B3")
                    .attr("stroke-width",pathStrokeWidth)
                    .attr("fill","none");

                 alarmsImage = d3.select(aCircle)
                    .selectAll('svg')
                    .append("image")
                     .attr("id", "alarms_image_default")
                    .attr("xlink:href", "/module/sbcOfflineEngineering/rc/images/scan_alarms_default.svg")
                    .attr("x", imgx)
                    .attr("y", imgy)
                    .attr("height", imgheight)
                    .attr("width", imgwidth);

                  //9 - here is the height of the text
                  //5 - here half of the width of the text
                 alarmsText = d3.select(aCircle)
                    .selectAll('svg')
                    .append("text")
                    .attr("x",(x-5))
                    .attr("y",(y+(radius-25)))
                    .text("0")
                    .attr("fill","#222222");

                hdlogsCircle = hdlogsSVG.append('circle').attr('cx', x)
                .attr('cy',y)
                .attr('r',radius)
                .attr('fill','none')
                .attr('stroke','#CDCFD1')
                .attr('stroke-width',circleStrokeWidth);

            hdlogsPath = hdlogsSVG.append("path")
                .attr("stroke","#0066B3")
                .attr("stroke-width",pathStrokeWidth)
                .attr("fill","none");

            hdlogsImage = d3.select(hCircle)
                .selectAll('svg')
                .append("image")
                .attr("id", "hdlogs_image_default")
                .attr("xlink:href", "/module/sbcOfflineEngineering/rc/images/scan_hd_logs_default.svg")
                .attr("x", imgx)
                .attr("y", imgy)
                .attr("height", imgheight)
                .attr("width", imgwidth);

            hdlogsText = d3.select(hCircle)
                .selectAll('svg')
                .append("text")
                .attr("x",(x-5))
                .attr("y",(y+(radius-25)))
                .text("0")
                .attr("fill","#222222");

            _validateImageDimensions();

        };

        Progress.prototype.buildAtFinish = function(dCircle, aCircle, hCircle, dPath, aPath, hPath){

            //Datapoints status icon
            d3.select(dCircle).append("svg").attr("viewBox", "0 0 240 240");
            d3.select(dCircle)
                    .selectAll('svg')
                    .append("image")
                    .attr("id", "datapoints_image_finish")
                    .attr("xlink:href", dPath)
                    .attr("x", imgx)
                    .attr("y", imgy)
                    .attr("height", imgheight)
                    .attr("width", imgwidth);


            //Alarms status icon
            d3.select(aCircle).append("svg").attr("viewBox", "0 0 240 240");
            d3.select(aCircle)
                    .selectAll('svg')
                    .append("image")
                    .attr("id", "alarms_image_finish")
                    .attr("xlink:href", aPath)
                    .attr("x", imgx)
                    .attr("y", imgy)
                    .attr("height", imgheight)
                    .attr("width", imgwidth);


            //HDLogs status icon
            d3.select(hCircle).append("svg").attr("viewBox", "0 0 240 240");
            d3.select(hCircle)
                    .selectAll('svg')
                    .append("image")
                    .attr("id", "hdlogs_image_finish")
                    .attr("xlink:href", hPath)
                    .attr("x", imgx)
                    .attr("y", imgy)
                    .attr("height", imgheight)
                    .attr("width", imgwidth);

            d3.select("#datapoints_image_finish").attr("width", "80%").attr("height", "80%");
            d3.select("#alarms_image_finish").attr("width", "80%").attr("height", "80%");
            d3.select("#hdlogs_image_finish").attr("width", "80%").attr("height", "80%");
        };

        Progress.prototype.setAlarmsCount = function(alarmsCount) {
                alarmsText.text(alarmsCount);
                alarmsText.attr("x",(x-((alarmsCount.toString().length*5)-5)));
        };

        Progress.prototype.setDatapointsCount = function(datapointsCount) {
                datapointsText.text(datapointsCount);
                datapointsText.attr("x",(x-((datapointsCount.toString().length*5)-5)));
        };

        Progress.prototype.setHDLogsCount = function(hdlogsCount) {
            hdlogsText.text(hdlogsCount);
            hdlogsText.attr("x",(x-((hdlogsCount.toString().length*5)-5)));
        };



    return Progress;
});
