/**
 * * Copyright (c) 2018. Honeywell International, Inc. All Rights Reserved.
 */
define([], function () {
    'use strict';

    var SearchFilter = function (query, ignoreCase) {
        if (ignoreCase === undefined) {
            ignoreCase = true;
        }
        var sanitizedQuery = this.sanitizeQuery(query);
        this.regex = ignoreCase === true ? new RegExp(sanitizedQuery, "i") : new RegExp(sanitizedQuery);
    }

    SearchFilter.prototype.filter = function (array, searchKey) {
        var regex = this.regex, refinedArray = [];
        for (var i=0; i<array.length;i++) {
            if(regex.test(array[i][searchKey])){
                refinedArray.push(array[i]);
            }
        }
        return refinedArray;
    }

    SearchFilter.prototype.sanitizeQuery = function(queryString) {
        if (queryString === undefined) {
            return undefined;
        }
        var sanitizedQuery = queryString.replace(/\\/g, "\\\\"); //remove special meaning of '\'
        sanitizedQuery = sanitizedQuery.replace(/\./g, "\\."); //remove special meaning of dot
        sanitizedQuery = sanitizedQuery.replace(/\(/g, "\\("); //remove special meaning of '('
        sanitizedQuery = sanitizedQuery.replace(/\[/g, "\\["); //remove special meaning of '['
        sanitizedQuery = sanitizedQuery.replace(/\)/g, "\\)"); //remove special meaning of ')'
        sanitizedQuery = sanitizedQuery.replace(/\*/g, ".*"); // make * matcher for any character
        return sanitizedQuery;
    }

    return SearchFilter;
});